from __future__ import annotations

import redis
import pymongo
import psycopg2
from typing import AsyncGenerator
from loguru import logger
from sqlalchemy.ext.asyncio import AsyncEngine, AsyncSession, async_sessionmaker, create_async_engine
from service_forge.service_config import ServiceConfig
from pymongo import AsyncMongoClient

class PostgresDatabase:
    def __init__(
        self,
        name: str,
        postgres_user: str,
        postgres_password: str,
        postgres_host: str,
        postgres_port: int,
        postgres_db: str,
    ) -> None:
        self.name = name
        self.postgres_user = postgres_user
        self.postgres_password = postgres_password
        self.postgres_host = postgres_host
        self.postgres_port = postgres_port
        self.postgres_db = postgres_db
        self.engine = None
        self.session_factory = None
        self.test_connection()

    @property
    def database_url(self) -> str:
        return f"postgresql+asyncpg://{self.postgres_user}:{self.postgres_password}@{self.postgres_host}:{self.postgres_port}/{self.postgres_db}"

    @property
    def database_base_url(self) -> str:
        return f"postgresql+asyncpg://{self.postgres_user}:{self.postgres_password}@{self.postgres_host}:{self.postgres_port}/postgres"

    async def init(self) -> None:
        if self.engine is None:
            self.engine = await self.create_engine()
            self.session_factory = async_sessionmaker(bind=self.engine, class_=AsyncSession, expire_on_commit=False)

    async def close(self) -> None:
        if self.engine:
            await self.engine.dispose()
            self.engine = None
            self.session_factory = None
            logger.info("Database connection closed")

    async def create_engine(self) -> AsyncEngine:
        if not all([self.postgres_user, self.postgres_host, self.postgres_port, self.postgres_db]):
            raise ValueError("Missing required database configuration. Please check your .env file or configuration.")
        logger.info(f"Creating database engine: {self.database_url}")
        return create_async_engine(self.database_url)

    async def get_async_session(self) -> AsyncGenerator[AsyncSession, None]:
        if self.session_factory is None:
            await self.init()
        
        if self.session_factory is None:
            raise RuntimeError("Session factory is not initialized")
        
        async with self.session_factory() as session:
            try:
                yield session
            except Exception:
                await session.rollback()
                raise
            finally:
                await session.close()
            yield session

    async def get_session_factory(self) -> async_sessionmaker[AsyncSession]:
        if self.engine is None:
            await self.init()

        if self.session_factory is None:
            raise RuntimeError("Session factory is not initialized")

        return self.session_factory

    def test_connection(self) -> bool:
        try:
            conn = psycopg2.connect(
                host=self.postgres_host,
                port=self.postgres_port,
                user=self.postgres_user,
                password=self.postgres_password,
                database=self.postgres_db,
                connect_timeout=5
            )
            conn.close()
            logger.info(f"PostgreSQL connection test successful for database '{self.name}'")
            return True
        except Exception as e:
            logger.warning(f"PostgreSQL connection test failed for database '{self.name}': {e}")
            return False

class MongoDatabase:
    def __init__(
        self,
        name: str,
        mongo_host: str,
        mongo_port: int,
        mongo_user: str,
        mongo_password: str,
        mongo_db: str,
    ) -> None:
        self.name = name
        self.mongo_host = mongo_host
        self.mongo_port = mongo_port
        self.mongo_user = mongo_user
        self.mongo_password = mongo_password
        self.mongo_db = mongo_db or ""
        self.client = pymongo.MongoClient(self.database_url)
        self.async_client = AsyncMongoClient(self.database_url)
        self.test_connection()

    @property
    def database_url(self) -> str:
        return f"mongodb://{self.mongo_user}:{self.mongo_password}@{self.mongo_host}:{self.mongo_port}/{self.mongo_db}"

    def test_connection(self) -> bool:
        try:
            self.client.admin.command('ping')
            logger.info(f"MongoDB connection test successful for database '{self.name}'")
            return True
        except Exception as e:
            logger.error(f"MongoDB connection test failed for database '{self.name}': {e}")
            return False

    async def test_async_connection(self) -> bool:
        try:
            await self.async_client.admin.command('ping')
            logger.info(f"Async MongoDB connection test successful for database '{self.name}'")
            return True
        except Exception as e:
            logger.error(f"Async MongoDB connection test failed for database '{self.name}': {e}")
            return False

    def get_sync_collection(self, collection_name: str):
        return self.client[self.mongo_db][collection_name]

    def get_async_collection(self, collection_name: str):
        return self.async_client[self.mongo_db][collection_name]

class RedisDatabase:
    def __init__(
        self,
        name: str,
        redis_host: str,
        redis_port: int,
        redis_password: str,
    ) -> None:
        self.name = name
        self.redis_host = redis_host
        self.redis_port = redis_port
        self.redis_password = redis_password
        self.client = redis.Redis(host=redis_host, port=redis_port, password=redis_password)
        self.test_connection()

    def test_connection(self) -> bool:
        try:
            self.client.ping()
            logger.info(f"Redis connection test successful for database '{self.name}'")
            return True
        except Exception as e:
            logger.error(f"Redis connection test failed for database '{self.name}': {e}")
            return False


class DatabaseManager:
    def __init__(
        self,
        postgres_databases: list[PostgresDatabase],
        mongo_databases: list[MongoDatabase],
        redis_databases: list[RedisDatabase],
    ) -> None:
        self.postgres_databases = postgres_databases
        self.mongo_databases = mongo_databases
        self.redis_databases = redis_databases

    def get_database(self, name: str) ->  PostgresDatabase | MongoDatabase | RedisDatabase | None:
        for database in self.postgres_databases:
            if database.name == name:
                return database
        return None

    def get_default_postgres_database(self) -> PostgresDatabase | None:
        if len(self.postgres_databases) > 0:
            return self.postgres_databases[0]
        return None

    def get_default_mongo_database(self) -> MongoDatabase | None:
        if len(self.mongo_databases) > 0:
            return self.mongo_databases[0]
        return None

    def get_default_redis_database(self) -> RedisDatabase | None:
        if len(self.redis_databases) > 0:
            return self.redis_databases[0]
        return None

    @staticmethod
    def from_config(config_path: str = None, config: ServiceConfig = None) -> DatabaseManager:
        if config is None:
            config = ServiceConfig.from_yaml_file(config_path)

        postgres_databases = []
        mongo_databases = []
        redis_databases = []

        databases_config = config.databases

        if databases_config is not None:
            for database_config in databases_config:
                if all([database_config.postgres_host is None, database_config.mongo_host is None, database_config.redis_host is None]):
                    raise ValueError(f"Database '{database_config.name}' is missing required configuration. Please check your service.yaml file.")

                if (database_config.postgres_host is not None) + (database_config.mongo_host is not None) + (database_config.redis_host is not None) > 1:
                    raise ValueError(f"Database '{database_config['name']}' has multiple host configurations. Please check your service.yaml file.")

                if database_config.postgres_host is not None:
                    postgres_databases.append(PostgresDatabase(
                        name=database_config.name,
                        postgres_user=database_config.postgres_user,
                        postgres_password=database_config.postgres_password,
                        postgres_host=database_config.postgres_host,
                        postgres_port=database_config.postgres_port,
                        postgres_db=database_config.postgres_db,
                    ))
                elif database_config.mongo_host is not None:
                    mongo_databases.append(MongoDatabase(
                        name=database_config.name,
                        mongo_host=database_config.mongo_host,
                        mongo_port=database_config.mongo_port,
                        mongo_user=database_config.mongo_user,
                        mongo_password=database_config.mongo_password,
                        mongo_db=database_config.mongo_db,
                    ))
                elif database_config.redis_host is not None:
                    redis_databases.append(RedisDatabase(
                        name=database_config.name,
                        redis_host=database_config.redis_host,
                        redis_port=database_config.redis_port,
                        redis_password=database_config.redis_password,
                    ))

        return DatabaseManager(postgres_databases=postgres_databases, mongo_databases=mongo_databases, redis_databases=redis_databases)

def create_database_manager(config_path: str = None, config = None) -> DatabaseManager:
    return DatabaseManager.from_config(config_path, config)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: entryService/record.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'entryService/record.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x65ntryService/record.proto\x12\x0c\x65ntryService\"\xbc\x02\n\x06Record\x12\x10\n\x08proto_id\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\r\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\x04text\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\x0f\n\x07user_id\x18\x06 \x01(\r\x12\x10\n\x08group_id\x18\x07 \x01(\r\x12\x0f\n\x07\x66ile_id\x18\x08 \x01(\t\x12\x15\n\ris_recognized\x18\t \x01(\x08\x12\x12\n\nis_deleted\x18\n \x01(\x08\x12\x11\n\ttimestamp\x18\x0b \x01(\t\x12\x0c\n\x04role\x18\x0c \x01(\t\x12\x11\n\tis_stream\x18\r \x01(\x08\x12\x16\n\x0eis_stream_done\x18\x0e \x01(\x08\x12\x13\n\x0b\x66orce_reply\x18\x0f \x01(\x08\x12\x0c\n\x04tags\x18\x10 \x03(\t\x12\x0e\n\x06status\x18\x11 \x01(\tB\x07\n\x05_textB?Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'entryService.record_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z=github.com/NextHCI/second-brain-backends-go/protobuf;protobuf'
  _globals['_RECORD']._serialized_start=44
  _globals['_RECORD']._serialized_end=360
# @@protoc_insertion_point(module_scope)



from __future__ import annotations
from loguru import logger
from service_forge.workflow.trigger import Trigger
from typing import AsyncIterator, Any
from service_forge.workflow.port import Port
from google.protobuf.message import Message
from google.protobuf.json_format import MessageToJson
from fastapi import FastAPI
from a2a.types import (
    AgentCapabilities,
    AgentCard,
    AgentSkill,
)
from a2a.server.apps import A2AStarletteApplication
from a2a.server.request_handlers import DefaultRequestHandler
from a2a.server.tasks import InMemoryTaskStore
from a2a.server.agent_execution import AgentExecutor, RequestContext
from a2a.server.events import EventQueue
from a2a.utils import new_agent_text_message
from a2a.utils.constants import DEFAULT_RPC_URL, EXTENDED_AGENT_CARD_PATH, AGENT_CARD_WELL_KNOWN_PATH

import json
import uuid
import asyncio
from service_forge.workflow.workflow_event import WorkflowResult

class A2AAgentExecutor(AgentExecutor):
    def __init__(self, trigger: A2AAPITrigger):
        self.trigger = trigger

    @staticmethod
    def serialize_result(result: Any) -> str:
        if isinstance(result, Message):
            return MessageToJson(
                result,
                preserving_proto_field_name=True
            )
        return json.dumps(result)

    async def send_event(self, event_queue: EventQueue, item: WorkflowResult) -> None:
        if item.is_error:
            result = {
                'event': 'error',
                'detail': str(item.result)
            }
            await event_queue.enqueue_event(new_agent_text_message(json.dumps(result)))

        if item.is_end:
            result = {
                'event': 'end',
                'detail': self.serialize_result(item.result)
            }
            await event_queue.enqueue_event(new_agent_text_message(json.dumps(result)))

        result = {
            'event': 'data',
            'data': self.serialize_result(item.result)
        }
        await event_queue.enqueue_event(new_agent_text_message(json.dumps(result)))

    async def execute(
        self,
        context: RequestContext,
        event_queue: EventQueue,
    ) -> None:
        task_id = uuid.uuid4()
        self.trigger.result_queues[task_id] = asyncio.Queue()

        self.trigger.trigger_queue.put_nowait({
            'id': task_id,
            'context': context,
        })

        # TODO: support stream output
        if False:
            self.trigger.stream_queues[task_id] = asyncio.Queue()
            while True:
                item = await self.trigger.stream_queues[task_id].get()
                await self.send_event(event_queue, item)

                if item.is_error or item.is_end:
                    break

            if task_id in self.trigger.stream_queues:
                del self.trigger.stream_queues[task_id]
        else:
            result = await self.trigger.result_queues[task_id].get()
            await self.send_event(event_queue, result)

        if task_id in self.trigger.result_queues:
            del self.trigger.result_queues[task_id]

    async def cancel(
        self, context: RequestContext, event_queue: EventQueue
    ) -> None:
        raise Exception('cancel not supported')


class A2AAPITrigger(Trigger):
    DEFAULT_INPUT_PORTS = [
        Port("app", FastAPI),
        Port("path", str),
        Port("skill_id", str, is_extended=True),
        Port("skill_name", str, is_extended=True),
        Port("skill_description", str, is_extended=True),
        Port("skill_tags", list[str], is_extended=True),
        Port("skill_examples", list[str], is_extended=True),
        Port("agent_name", str),
        Port("agent_url", str),
        Port("agent_description", str),
        Port("agent_version", str),
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("trigger", bool),
        Port("context", RequestContext),
    ]

    def __init__(self, name: str):
        super().__init__(name)
        self.events = {}
        self.is_setup_handler = False
        self.agent_card: AgentCard | None = None

    @staticmethod
    def serialize_result(result: Any):
        if isinstance(result, Message):
            return MessageToJson(
                result,
                preserving_proto_field_name=True
            )
        return result

    def _setup_handler(
        self,
        app: FastAPI,
        path: str,
        skill_id: list[tuple[int, str]],
        skill_name: list[tuple[int, str]],
        skill_description: list[tuple[int, str]],
        skill_tags: list[tuple[int, list[str]]],
        skill_examples: list[tuple[int, list[str]]],
        agent_name: str,
        agent_url: str,
        agent_description: str,
        agent_version: str,
    ) -> None:
    
        skills_config = []
        for i in range(len(skill_id)):
            skills_config.append({
                'id': '',
                'name': '',
                'description': '',
                'tags': [],
                'examples': [],
            })

        for i in range(len(skill_id)):
            skills_config[skill_id[i][0]]['id'] = skill_id[i][1]
            skills_config[skill_name[i][0]]['name'] = skill_name[i][1]
            skills_config[skill_description[i][0]]['description'] = skill_description[i][1]
            skills_config[skill_tags[i][0]]['tags'] = skill_tags[i][1]
            skills_config[skill_examples[i][0]]['examples'] = skill_examples[i][1]
    
        skills = []
        for config in skills_config:
            skills.append(AgentSkill(
                id=config['id'],
                name=config['name'],
                description=config['description'],
                tags=config['tags'],
                examples=config['examples'],
            ))

        agent_card = AgentCard(
            name=agent_name,
            description=agent_description,
            url=agent_url,
            version=agent_version,
            default_input_modes=['text'],
            default_output_modes=['text'],
            capabilities=AgentCapabilities(streaming=True),
            skills=skills,
            supports_authenticated_extended_card=False,
        )
        
        self.agent_card = agent_card

        request_handler = DefaultRequestHandler(
            agent_executor=A2AAgentExecutor(self),
            task_store=InMemoryTaskStore(),
        )

        try:
            server = A2AStarletteApplication(
                agent_card=agent_card,
                http_handler=request_handler,
            )
            
            server.add_routes_to_app(
                app,
                agent_card_url="/a2a" + path + AGENT_CARD_WELL_KNOWN_PATH,
                rpc_url="/a2a" + path + DEFAULT_RPC_URL,
                extended_agent_card_url="/a2a" + path + EXTENDED_AGENT_CARD_PATH,
            )

        except Exception as e:
            logger.error(f"Error adding A2A routes: {e}")
            raise

    async def _run(
        self,
        app: FastAPI,
        path: str,
        skill_id: list[tuple[int, str]],
        skill_name: list[tuple[int, str]],
        skill_description: list[tuple[int, str]],
        skill_tags: list[tuple[int, list[str]]],
        skill_examples: list[tuple[int, list[str]]],
        agent_name: str,
        agent_url: str,
        agent_description: str,
        agent_version: str,
    ) -> AsyncIterator[bool]:
        if len(skill_id) != len(skill_name) or len(skill_id) != len(skill_description) or len(skill_id) != len(skill_tags) or len(skill_id) != len(skill_examples):
            raise ValueError("skill_id, skill_name, skill_description, skill_tags, skill_examples must have the same length")

        if not self.is_setup_handler:
            self._setup_handler(
                app,
                path,
                skill_id,
                skill_name,
                skill_description,
                skill_tags,
                skill_examples,
                agent_name,
                agent_url,
                agent_description,
                agent_version,
            )
            self.is_setup_handler = True

        logger.info(f"A2A Trigger {self.name} is running")

        while True:
            try:
                trigger = await self.trigger_queue.get()
                self.prepare_output_edges('context', trigger['context'])
                yield self.trigger(trigger['id'])
            except Exception as e:
                logger.error(f"Error in A2AAPITrigger._run: {e}")
                continue

    async def _stop(self) -> AsyncIterator[bool]:
        pass



from ..utils.type_converter import TypeConverter
from ..workflow.workflow import Workflow
from ..api.http_api import fastapi_app
from ..api.kafka_api import KafkaApp, kafka_app
from ..workflow.workflow_type import WorkflowType, workflow_type_register
from fastapi import FastAPI

type_converter = TypeConverter()
type_converter.register(str, Workflow, lambda s, node: node.sub_workflows.get_workflow(s))
type_converter.register(str, FastAPI, lambda s, node: fastapi_app)
type_converter.register(str, KafkaApp, lambda s, node: kafka_app)
type_converter.register(str, type, lambda s, node: workflow_type_register.items[s].type)


from __future__ import annotations

from typing import Any

class WorkflowResult:
    def __init__(
        self,
        result: Any | None,
        is_end: bool,
        is_error: bool,
    ) -> None:
        # when is_end is True, result is from the output port of the workflow
        self.result = result
        self.is_end = is_end
        self.is_error = is_error


from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class Tag(_message.Message):
    __slots__ = ("proto_id", "record_id", "user_id", "tags")
    PROTO_ID_FIELD_NUMBER: _ClassVar[int]
    RECORD_ID_FIELD_NUMBER: _ClassVar[int]
    USER_ID_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    proto_id: str
    record_id: int
    user_id: int
    tags: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, proto_id: _Optional[str] = ..., record_id: _Optional[int] = ..., user_id: _Optional[int] = ..., tags: _Optional[_Iterable[str]] = ...) -> None: ...



import asyncio
import json
import sys
import uuid

from asyncpg import InvalidCatalogNameError
from loguru import logger
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy import text

from alembic import command
from alembic.config import Config

from service_forge.db.database import DatabaseManager, PostgresDatabase, MongoDatabase, RedisDatabase

async def ensure_database_exists(database: PostgresDatabase):
    target_url = database.database_url
    base_url = database.database_base_url

    logger.info(f"Checking database: {database.postgres_db}")
    
    try:
        engine = create_async_engine(target_url)
        async with engine.begin() as conn:
            logger.info(f"Database {database.postgres_db} already exists")
        await engine.dispose()
    except InvalidCatalogNameError:
        logger.info(f"Creating database: {database.postgres_db}")
        try:
            admin_engine = create_async_engine(base_url, isolation_level="AUTOCOMMIT")
            async with admin_engine.connect() as conn:
                await conn.execute(text(f'CREATE DATABASE "{database.postgres_db}"'))
            await admin_engine.dispose()
            logger.info(f"Database {database.postgres_db} created successfully")
        except Exception as e:
            logger.error(f"Failed to create database {database.postgres_db}: {e}")
            sys.exit(1)


async def run_migrations(database: PostgresDatabase) -> bool:
    try:
        logger.info("Starting database migrations...")
        
        from sqlalchemy.ext.asyncio import create_async_engine
        
        async_engine = create_async_engine(database.database_url)
        async with async_engine.begin() as conn:
            alembic_cfg = Config("alembic.ini")
            
            def run_upgrade(connection, cfg):
                cfg.attributes["connection"] = connection
                command.upgrade(cfg, "head")
            
            await conn.run_sync(run_upgrade, alembic_cfg)
        
        await async_engine.dispose()
        logger.info("Database migrations completed successfully")
        return True
    except Exception as e:
        logger.error(f"Database migrations failed: {e}")
        return False
        
async def update_default_tag(database: PostgresDatabase):
    tags = json.load(open("prompt/default_tag.json"))

    session_factory = await database.get_session_factory()

    async with session_factory() as session:
        tag_names = [tag['name'] for tag in tags['tags']]
        if tag_names:
            placeholders = ','.join([f':name_{i}' for i in range(len(tag_names))])
            params = {f'name_{i}': name for i, name in enumerate(tag_names)}
            await session.execute(
                text(f"DELETE FROM tag WHERE type = 'DEFAULT' AND name NOT IN ({placeholders})"),
                params
            )
        else:
            await session.execute(
                text("DELETE FROM tag WHERE type = 'DEFAULT'")
            )
        await session.commit()

        for tag in tags['tags']:
            result = await session.execute(text("SELECT * FROM tag WHERE name = :name"), {"name": tag['name']})
            if result.fetchone() is None:
                await session.execute(
                    text("INSERT INTO tag (id, name, type, description, example, created_at, updated_at) VALUES (:id, :name, 'DEFAULT', :description, :example, now(), now())"),
                    {"id": str(uuid.uuid4()), "name": tag['name'], "description": tag['description'], "example": tag['example']}
                )
                await session.commit()
                print(f"Tag {tag['name']} created")
            else:
                await session.execute(
                    text("UPDATE tag SET description = :description, example = :example, updated_at = now() WHERE name = :name AND type = 'DEFAULT'"),
                    {"name": tag['name'], "description": tag['description'], "example": tag['example']}
                )
                await session.commit()
                print(f"Tag {tag['name']} updated")

async def main():
    logger.info("Starting database migration process")

    database_manager = DatabaseManager.from_config(config_path="configs/service.yaml")
    database = database_manager.get_default_postgres_database()
    if database is None:
        logger.error("No database found")
        return

    await ensure_database_exists(database)
    success = await run_migrations(database)
    await update_default_tag(database)
    
    if success:
        logger.info("Migration process completed successfully")
    else:
        logger.error("Migration process failed")

if __name__ == '__main__':
    asyncio.run(main())


from .get_tags_node import GetTagsNode
from .http_query_tags_node import HttpQueryTagsNode
from .http_create_tag_node import HttpCreateTagNode
from .http_update_tag_node import HttpUpdateTagNode
from .http_delete_tag_node import HttpDeleteTagNode
from .test_sse_node import TestSSENode
from .test_context_node import TestContextNode
from .test_a2a_node import TestA2ANode


from service_forge.workflow.node import Node
from service_forge.workflow.port import Port
from model.http.update_tag_model import UpdateTagModel
from model.db.tag_base import TagBase
from sqlalchemy import select
from fastapi import HTTPException

class HttpUpdateTagNode(Node):
    DEFAULT_INPUT_PORTS = [
        Port("user_id", int),
        Port("tag", UpdateTagModel)
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("tag", TagBase)
    ]

    def __init__(self, name: str):
        super().__init__(name)

    async def _run(self, tag: UpdateTagModel, user_id: int) -> None:
        session_factory = await self.default_postgres_database.get_session_factory()
        async with session_factory() as session:
            try:
                result = await session.execute(
                    select(TagBase).where(TagBase.id == tag.id, TagBase.user_id == user_id)
                )
                db_tag = result.scalar_one_or_none()
                if db_tag is None:
                    self.activate_output_edges(self.get_output_port_by_name('tag'), HTTPException(status_code=404, detail="Tag not found"))
                    return
                for key, value in tag.model_dump().items():
                    setattr(db_tag, key, value)
                await session.commit()
            except Exception as e:
                self.activate_output_edges(self.get_output_port_by_name('tag'), HTTPException(status_code=500, detail="Tag not found"))
                return

        self.activate_output_edges(self.get_output_port_by_name('tag'), db_tag)


from __future__ import annotations
from typing import Any
from ...node import Node
from ...port import Port

class SwitchNode(Node):
    DEFAULT_INPUT_PORTS = [
        Port("TRIGGER", bool),
        Port("condition", str, is_extended=True),
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("result", Any, is_extended=True),
    ]

    def __init__(
        self, 
        name: str,
    ) -> None:
        super().__init__(
            name,
        )

    async def _run(self, condition: list[tuple[int, str]]) -> None:
        for index, cond in condition:
            if eval(cond):
                self.activate_output_edges(self.extended_output_name('result', index), str(index))
                break



from __future__ import annotations
import uuid
import asyncio
import json
from loguru import logger
from service_forge.workflow.trigger import Trigger
from typing import AsyncIterator, Any, Optional
from fastapi import FastAPI, Request
from fastapi.responses import StreamingResponse
from service_forge.workflow.port import Port
from service_forge.utils.default_type_converter import type_converter
from service_forge.api.routers.websocket.websocket_manager import websocket_manager
from fastapi import HTTPException
from google.protobuf.message import Message
from google.protobuf.json_format import MessageToJson

class FastAPITrigger(Trigger):
    DEFAULT_INPUT_PORTS = [
        Port("app", FastAPI),
        Port("path", str),
        Port("method", str),
        Port("data_type", type),
        Port("is_stream", bool),
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("trigger", bool),
        Port("user_id", int),
        Port("token", str),
        Port("data", Any),
        Port("path_params", dict),
    ]

    def __init__(self, name: str):
        super().__init__(name)
        self.events = {}
        self.is_setup_route = False
        self.app = None
        self.route_path = None
        self.route_method = None

    @staticmethod
    def serialize_result(result: Any):
        if isinstance(result, Message):
            return MessageToJson(
                result,
                preserving_proto_field_name=True
            )
        return result

    async def handle_request(
        self,
        request: Request,
        data_type: type,
        extract_data_fn: callable[[Request], dict],
        is_stream: bool,
        path_params: Optional[dict] = None,
    ):
        task_id = uuid.uuid4()
        self.result_queues[task_id] = asyncio.Queue()

        body_data = await extract_data_fn(request)
        # Merge path parameters into body_data (path params take precedence)
        if path_params:
            body_data = {**body_data, **path_params}
        converted_data = data_type(**body_data)

        client_id = (
            body_data.get("client_id")
            or request.query_params.get("client_id")
            or request.headers.get("X-Client-ID")
        )
        if client_id:
            workflow_name = getattr(self.workflow, "name", "Unknown")
            steps = len(self.workflow.nodes) if hasattr(self.workflow, "nodes") else 1
            websocket_manager.create_task_with_client(task_id, client_id, workflow_name, steps)

        self.trigger_queue.put_nowait({
            "id": task_id,
            "user_id": getattr(request.state, "user_id", None),
            "token": getattr(request.state, "auth_token", None),
            "data": converted_data,
            "path_params": path_params,
        })

        if is_stream:
            self.stream_queues[task_id] = asyncio.Queue()

            async def generate_sse():
                try:
                    while True:
                        item = await self.stream_queues[task_id].get()

                        if item.is_error:
                            yield f"event: error\ndata: {json.dumps({'detail': str(item.result)})}\n\n"
                            break
                        
                        if item.is_end:
                            # TODO: send the result?
                            break

                        # TODO: modify
                        serialized = self.serialize_result(item.result)
                        if isinstance(serialized, str):
                            data = serialized
                        else:
                            data = json.dumps(serialized)
                        
                        yield f"data: {data}\n\n"
                    
                except Exception as e:
                    yield f"event: error\ndata: {json.dumps({'detail': str(e)})}\n\n"
                finally:
                    if task_id in self.stream_queues:
                        del self.stream_queues[task_id]
                    if task_id in self.result_queues:
                        del self.result_queues[task_id]
            
            return StreamingResponse(
                generate_sse(),
                media_type="text/event-stream",
                headers={
                    "Cache-Control": "no-cache",
                    "Connection": "keep-alive",
                    "X-Accel-Buffering": "no",
                }
            )
        else:
            result = await self.result_queues[task_id].get()
            del self.result_queues[task_id]

            if result.is_error:
                if isinstance(result.result, HTTPException):
                    raise result.result
                else:
                    raise HTTPException(status_code=500, detail=str(result.result))

            return self.serialize_result(result.result)

    def _setup_route(self, app: FastAPI, path: str, method: str, data_type: type, is_stream: bool) -> None:
        async def get_data(request: Request) -> dict:
            return dict(request.query_params)

        async def body_data(request: Request) -> dict:
            raw = await request.body()
            if not raw:
                return {}
            return json.loads(raw.decode("utf-8"))

        extractor = get_data if method == "GET" else body_data

        async def handler(request: Request):
            # Get path parameters from FastAPI request
            # request.path_params is always available in FastAPI and contains path parameters
            path_params = dict(request.path_params)
            return await self.handle_request(request, data_type, extractor, is_stream, path_params)

        # Save route information for cleanup
        self.app = app
        self.route_path = path
        self.route_method = method.upper()

        if method == "GET":
            app.get(path)(handler)
        elif method == "POST":
            app.post(path)(handler)
        elif method == "PUT":
            app.put(path)(handler)
        elif method == "DELETE":
            app.delete(path)(handler)
        else:
            raise ValueError(f"Invalid method {method}")

    async def _run(
        self, 
        app: FastAPI, 
        path: str, 
        method: str, 
        data_type: type, 
        is_stream: bool = False,
    ) -> AsyncIterator[bool]:
        if not self.is_setup_route:
            self._setup_route(app, path, method, data_type, is_stream)
            self.is_setup_route = True

        while True:
            try:
                trigger = await self.trigger_queue.get()
                self.prepare_output_edges(self.get_output_port_by_name('user_id'), trigger['user_id'])
                self.prepare_output_edges(self.get_output_port_by_name('token'), trigger['token'])
                self.prepare_output_edges(self.get_output_port_by_name('data'), trigger['data'])
                self.prepare_output_edges(self.get_output_port_by_name('path_params'), trigger['path_params'])
                yield self.trigger(trigger['id'])
            except Exception as e:
                logger.error(f"Error in FastAPITrigger._run: {e}")
                continue

    async def _stop(self) -> AsyncIterator[bool]:
        if self.is_setup_route:
            # Remove the route from the app
            if self.app and self.route_path and self.route_method:
                # Find and remove matching route
                routes_to_remove = []
                for route in self.app.routes:
                    if hasattr(route, "path") and hasattr(route, "methods"):
                        if route.path == self.route_path and self.route_method in route.methods:
                            routes_to_remove.append(route)
                
                # Remove found routes
                for route in routes_to_remove:
                    try:
                        self.app.routes.remove(route)
                        logger.info(f"Removed route {self.route_method} {self.route_path} from FastAPI app")
                    except ValueError:
                        logger.warning(f"Route {self.route_method} {self.route_path} not found in app.routes")
            
            # Reset route information
            self.app = None
            self.route_path = None
            self.route_method = None
            self.is_setup_route = False


from typing import Optional, Any
from datetime import datetime
from uuid import uuid4
from loguru import logger
from sqlalchemy import select, and_
from sqlalchemy.exc import SQLAlchemyError

from ..db.models.feedback import FeedbackBase
from ..db.database import DatabaseManager


class FeedbackStorage:
    """反馈存储管理器 - 使用 PostgreSQL 数据库存储"""

    def __init__(self, database_manager: DatabaseManager = None):
        self.database_manager = database_manager
        self._db = None
        # 内存存储后备(用于数据库未配置时)
        self._storage: dict[str, dict[str, Any]] = {}
        self._task_index: dict[str, list[str]] = {}
        self._workflow_index: dict[str, list[str]] = {}

    @property
    def db(self):
        """延迟获取数据库连接"""
        if self._db is None and self.database_manager is not None:
            self._db = self.database_manager.get_default_postgres_database()
        return self._db

    async def create_feedback(
        self,
        task_id: str,
        workflow_name: str,
        rating: Optional[int] = None,
        comment: Optional[str] = None,
        metadata: Optional[dict[str, Any]] = None,
    ) -> dict[str, Any]:
        """创建新的反馈记录"""
        if self.db is None:
            logger.warning("数据库未初始化,使用内存存储(数据将在重启后丢失)")
            return self._create_feedback_in_memory(task_id, workflow_name, rating, comment, metadata)

        try:
            feedback_id = uuid4()
            created_at = datetime.now()

            feedback = FeedbackBase(
                feedback_id=feedback_id,
                task_id=task_id,
                workflow_name=workflow_name,
                rating=rating,
                comment=comment,
                extra_metadata=metadata or {},
                created_at=created_at,
            )

            session_factory = await self.db.get_session_factory()
            async with session_factory() as session:
                session.add(feedback)
                await session.commit()
                await session.refresh(feedback)

                logger.info(f"创建反馈: feedback_id={feedback_id}, task_id={task_id}, workflow={workflow_name}")
                return feedback.to_dict()

        except SQLAlchemyError as e:
            logger.error(f"数据库创建反馈失败: {e}")
            raise
        except Exception as e:
            logger.error(f"创建反馈失败: {e}")
            raise

    async def get_feedback(self, feedback_id: str) -> Optional[dict[str, Any]]:
        """根据反馈ID获取反馈"""
        if self.db is None:
            return self._get_feedback_from_memory(feedback_id)

        try:
            session_factory = await self.db.get_session_factory()
            async with session_factory() as session:
                result = await session.execute(
                    select(FeedbackBase).where(FeedbackBase.feedback_id == feedback_id)
                )
                feedback = result.scalar_one_or_none()
                return feedback.to_dict() if feedback else None

        except SQLAlchemyError as e:
            logger.error(f"查询反馈失败: {e}")
            return None

    async def get_feedbacks_by_task(self, task_id: str) -> list[dict[str, Any]]:
        """根据任务ID获取所有反馈"""
        if self.db is None:
            return self._get_feedbacks_by_task_from_memory(task_id)

        try:
            session_factory = await self.db.get_session_factory()
            async with session_factory() as session:
                result = await session.execute(
                    select(FeedbackBase)
                    .where(FeedbackBase.task_id == task_id)
                    .order_by(FeedbackBase.created_at.desc())
                )
                feedbacks = result.scalars().all()
                return [f.to_dict() for f in feedbacks]

        except SQLAlchemyError as e:
            logger.error(f"查询任务反馈失败: {e}")
            return []

    async def get_feedbacks_by_workflow(self, workflow_name: str) -> list[dict[str, Any]]:
        """根据工作流名称获取所有反馈"""
        if self.db is None:
            return self._get_feedbacks_by_workflow_from_memory(workflow_name)

        try:
            session_factory = await self.db.get_session_factory()
            async with session_factory() as session:
                result = await session.execute(
                    select(FeedbackBase)
                    .where(FeedbackBase.workflow_name == workflow_name)
                    .order_by(FeedbackBase.created_at.desc())
                )
                feedbacks = result.scalars().all()
                return [f.to_dict() for f in feedbacks]

        except SQLAlchemyError as e:
            logger.error(f"查询工作流反馈失败: {e}")
            return []

    async def get_all_feedbacks(self) -> list[dict[str, Any]]:
        """获取所有反馈"""
        if self.db is None:
            return self._get_all_feedbacks_from_memory()

        try:
            session_factory = await self.db.get_session_factory()
            async with session_factory() as session:
                result = await session.execute(
                    select(FeedbackBase).order_by(FeedbackBase.created_at.desc())
                )
                feedbacks = result.scalars().all()
                return [f.to_dict() for f in feedbacks]

        except SQLAlchemyError as e:
            logger.error(f"查询所有反馈失败: {e}")
            return []

    async def delete_feedback(self, feedback_id: str) -> bool:
        """删除反馈"""
        if self.db is None:
            return self._delete_feedback_from_memory(feedback_id)

        try:
            session_factory = await self.db.get_session_factory()
            async with session_factory() as session:
                result = await session.execute(
                    select(FeedbackBase).where(FeedbackBase.feedback_id == feedback_id)
                )
                feedback = result.scalar_one_or_none()

                if not feedback:
                    return False

                await session.delete(feedback)
                await session.commit()

                logger.info(f"删除反馈: feedback_id={feedback_id}")
                return True

        except SQLAlchemyError as e:
            logger.error(f"删除反馈失败: {e}")
            return False

    # ========== 内存存储后备方法(用于数据库未配置时) ==========

    def _create_feedback_in_memory(
        self,
        task_id: str,
        workflow_name: str,
        rating: Optional[int],
        comment: Optional[str],
        metadata: Optional[dict[str, Any]],
    ) -> dict[str, Any]:
        """内存存储版本"""
        feedback_id = str(uuid4())
        created_at = datetime.now()

        feedback_data = {
            "feedback_id": feedback_id,
            "task_id": task_id,
            "workflow_name": workflow_name,
            "rating": rating,
            "comment": comment,
            "metadata": metadata or {},
            "created_at": created_at,
        }

        self._storage[feedback_id] = feedback_data

        if task_id not in self._task_index:
            self._task_index[task_id] = []
        self._task_index[task_id].append(feedback_id)

        if workflow_name not in self._workflow_index:
            self._workflow_index[workflow_name] = []
        self._workflow_index[workflow_name].append(feedback_id)

        logger.info(f"[内存存储]创建反馈: feedback_id={feedback_id}")
        return feedback_data

    def _get_feedback_from_memory(self, feedback_id: str) -> Optional[dict[str, Any]]:
        return self._storage.get(feedback_id)

    def _get_feedbacks_by_task_from_memory(self, task_id: str) -> list[dict[str, Any]]:
        feedback_ids = self._task_index.get(task_id, [])
        return [self._storage[fid] for fid in feedback_ids if fid in self._storage]

    def _get_feedbacks_by_workflow_from_memory(self, workflow_name: str) -> list[dict[str, Any]]:
        feedback_ids = self._workflow_index.get(workflow_name, [])
        return [self._storage[fid] for fid in feedback_ids if fid in self._storage]

    def _get_all_feedbacks_from_memory(self) -> list[dict[str, Any]]:
        return list(self._storage.values())

    def _delete_feedback_from_memory(self, feedback_id: str) -> bool:
        if feedback_id not in self._storage:
            return False

        feedback = self._storage[feedback_id]
        task_id = feedback["task_id"]
        workflow_name = feedback["workflow_name"]

        if task_id in self._task_index:
            self._task_index[task_id].remove(feedback_id)
        if workflow_name in self._workflow_index:
            self._workflow_index[workflow_name].remove(feedback_id)

        del self._storage[feedback_id]
        logger.info(f"[内存存储]删除反馈: feedback_id={feedback_id}")
        return True


# 全局单例实例
feedback_storage = FeedbackStorage()



import typer
from pathlib import Path
from typing import Callable, TypeVar, Any
from service_forge.sft.util.logger import log_error, log_info

T = TypeVar('T')

def assert_dir_exists(path: Path) -> None:
    if not path.exists():
        log_error(f"Directory does not exist: {path}")
        raise typer.Exit(1)
    if not path.is_dir():
        log_error(f"Path is not a directory: {path}")
        raise typer.Exit(1)
    log_info(f"Directory exists: {path}")

def assert_file_exists(path: Path) -> None:
    if not path.exists():
        log_error(f"File does not exist: {path}")
        raise typer.Exit(1)
    if not path.is_file():
        log_error(f"Path is not a file: {path}")
        raise typer.Exit(1)
    log_info(f"File exists: {path}")



from __future__ import annotations
import asyncio
from typing import AsyncIterator
from abc import ABC, abstractmethod
import uuid
from .node import Node
from .workflow_event import WorkflowResult

class Trigger(Node, ABC):
    def __init__(self, name: str):
        super().__init__(name)
        self.trigger_queue = asyncio.Queue()
        # for workflow result
        self.result_queues: dict[uuid.UUID, asyncio.Queue[WorkflowResult]] = {}
        # for node stream output
        self.stream_queues: dict[uuid.UUID, asyncio.Queue[WorkflowResult]] = {}

    @abstractmethod
    async def _run(self) -> AsyncIterator[bool]:
        ...

    @abstractmethod
    async def _stop(self) -> AsyncIterator[bool]:
        ...

    def trigger(self, task_id: uuid.UUID) -> bool:
        self.prepare_output_edges(self.get_output_port_by_name('trigger'), True)
        return task_id



from typing import Optional
import typer
from rich.console import Console
from rich.table import Table

from service_forge.sft.config.sft_config import SftConfig
from service_forge.sft.util.logger import log_error, log_info, log_success, log_warning
from service_forge.sft.config.sft_config import sft_config

def list_config() -> None:
    try:
        console = Console()
        
        table = Table(title="SFT Configuration", show_header=True, header_style="bold magenta")
        table.add_column("Key", style="cyan", no_wrap=True)
        table.add_column("Value", style="green")
        table.add_column("Description", style="yellow")
        
        # Automatically add rows for all config items
        config_dict = sft_config.to_dict()
        for key, value in sorted(config_dict.items()):
            description = SftConfig.CONFIG_DESCRIPTIONS.get(key, "No description available")
            table.add_row(key, str(value), description)
        
        console.print(table)
        console.print(f"\n[dim]Config file location: {sft_config.config_file_path}[/dim]")
    except Exception as e:
        log_error(f"Failed to load config: {e}")
        raise typer.Exit(1)

def get_config(key: str) -> None:
    try:
        value = sft_config.get(key)
        
        if value is None:
            log_error(f"Config key '{key}' not found")
            log_info("Available keys: config_root, sft_file_root, k8s_namespace")
            raise typer.Exit(1)
        
        log_info(f"{key} = {value}")
    except ValueError as e:
        log_error(str(e))
        raise typer.Exit(1)
    except Exception as e:
        log_error(f"Failed to get config: {e}")
        raise typer.Exit(1)

def set_config(key: str, value: str) -> None:
    try:
        current_value = sft_config.get(key)
        if current_value is None:
            log_error(f"Unknown config key: {key}")
            log_info("Available keys: config_root, sft_file_root, k8s_namespace")
            raise typer.Exit(1)
        
        sft_config.set(key, value)
        sft_config.save()
        
        log_success(f"Updated {key} = {value}")
        log_info(f"Config saved to {sft_config.config_file_path}")
    except ValueError as e:
        log_error(str(e))
        raise typer.Exit(1)
    except Exception as e:
        log_error(f"Failed to set config: {e}")
        raise typer.Exit(1)




#!/usr/bin/env python3
"""
Service Router 测试脚本
用于测试 service_router 的功能：
1. 查看 service 状态
2. 启动 workflow
3. 停止 workflow
4. 上传 workflow 配置文件并加载
"""
import asyncio
import json
import sys
import argparse
from pathlib import Path
from typing import Optional
import httpx


class ServiceRouterTester:
    def __init__(self, base_url: str = "http://localhost:37200"):
        self.base_url = base_url.rstrip('/')
        # 尝试两种可能的路径格式
        self.service_base_paths = [
            f"{self.base_url}/sdk/service",  # 如果没有 root_path
            f"{self.base_url}/api/v1/tag-service-0.0.1/sdk/service",  # 如果有 root_path
        ]
        self.service_path = None
    
    async def _find_service_path(self) -> bool:
        """尝试找到正确的 service API 路径"""
        async with httpx.AsyncClient(timeout=10.0) as client:
            for path in self.service_base_paths:
                try:
                    response = await client.get(f"{path}/status")
                    if response.status_code == 200:
                        self.service_path = path
                        print(f"✅ 找到 service API 路径: {path}")
                        return True
                except Exception as e:
                    continue
        return False
    
    async def get_service_status(self) -> Optional[dict]:
        """获取 service 状态"""
        if not self.service_path:
            if not await self._find_service_path():
                print("❌ 无法找到 service API 路径")
                return None
        
        async with httpx.AsyncClient(timeout=10.0) as client:
            try:
                response = await client.get(f"{self.service_path}/status")
                response.raise_for_status()
                return response.json()
            except httpx.HTTPStatusError as e:
                print(f"❌ HTTP 错误: {e.response.status_code} - {e.response.text}")
                return None
            except Exception as e:
                print(f"❌ 发生错误: {e}")
                return None
    
    async def start_workflow(self, workflow_name: str) -> bool:
        """启动指定的 workflow"""
        if not self.service_path:
            if not await self._find_service_path():
                print("❌ 无法找到 service API 路径")
                return False
        
        async with httpx.AsyncClient(timeout=10.0) as client:
            try:
                response = await client.post(f"{self.service_path}/workflow/{workflow_name}/start")
                response.raise_for_status()
                result = response.json()
                if result.get("success"):
                    print(f"✅ {result.get('message', 'Workflow started successfully')}")
                    return True
                else:
                    print(f"❌ {result.get('message', 'Failed to start workflow')}")
                    return False
            except httpx.HTTPStatusError as e:
                print(f"❌ HTTP 错误: {e.response.status_code} - {e.response.text}")
                return False
            except Exception as e:
                print(f"❌ 发生错误: {e}")
                return False
    
    async def stop_workflow(self, workflow_name: str) -> bool:
        """停止指定的 workflow"""
        if not self.service_path:
            if not await self._find_service_path():
                print("❌ 无法找到 service API 路径")
                return False
        
        async with httpx.AsyncClient(timeout=10.0) as client:
            try:
                response = await client.post(f"{self.service_path}/workflow/{workflow_name}/stop")
                response.raise_for_status()
                result = response.json()
                if result.get("success"):
                    print(f"✅ {result.get('message', 'Workflow stopped successfully')}")
                    return True
                else:
                    print(f"❌ {result.get('message', 'Failed to stop workflow')}")
                    return False
            except httpx.HTTPStatusError as e:
                print(f"❌ HTTP 错误: {e.response.status_code} - {e.response.text}")
                return False
            except Exception as e:
                print(f"❌ 发生错误: {e}")
                return False
    
    async def upload_workflow_config(self, config_path: str, workflow_name: Optional[str] = None) -> bool:
        """上传 workflow 配置文件并加载"""
        if not self.service_path:
            if not await self._find_service_path():
                print("❌ 无法找到 service API 路径")
                return False
        
        config_file = Path(config_path)
        if not config_file.exists():
            print(f"❌ 配置文件不存在: {config_path}")
            return False
        
        async with httpx.AsyncClient(timeout=30.0) as client:
            try:
                with open(config_file, 'rb') as f:
                    files = {'file': (config_file.name, f, 'application/x-yaml')}
                    data = {}
                    if workflow_name:
                        data['workflow_name'] = workflow_name
                    
                    response = await client.post(
                        f"{self.service_path}/workflow/upload",
                        files=files,
                        data=data
                    )
                    response.raise_for_status()
                    result = response.json()
                    if result.get("success"):
                        print(f"✅ {result.get('message', 'Workflow uploaded and loaded successfully')}")
                        return True
                    else:
                        print(f"❌ {result.get('message', 'Failed to upload workflow')}")
                        return False
            except httpx.HTTPStatusError as e:
                print(f"❌ HTTP 错误: {e.response.status_code} - {e.response.text}")
                return False
            except Exception as e:
                print(f"❌ 发生错误: {e}")
                return False


async def test_all(tester: ServiceRouterTester):
    """运行所有测试"""
    print("=" * 60)
    print("开始测试 Service Router 功能")
    print("=" * 60)
    
    # 1. 测试获取 service 状态
    print("\n📊 测试 1: 获取 service 状态")
    print("-" * 60)
    status = await tester.get_service_status()
    if status:
        print(f"Service 名称: {status.get('name')}")
        print(f"Service 版本: {status.get('version')}")
        print(f"Service 描述: {status.get('description')}")
        print(f"\nWorkflows:")
        for workflow in status.get('workflows', []):
            status_icon = "🟢" if workflow.get('status') == 'running' else "🔴"
            print(f"  {status_icon} {workflow.get('name')}: {workflow.get('status')} (配置: {workflow.get('config_path')})")
    else:
        print("❌ 无法获取 service 状态")
        return
    
    # 2. 测试停止 workflow（如果正在运行）
    print("\n🛑 测试 2: 停止 workflow")
    print("-" * 60)
    workflow_name = "query_tags_workflow"
    print(f"尝试停止 workflow: {workflow_name}")
    await tester.stop_workflow(workflow_name)
    await asyncio.sleep(1)  # 等待一下
    
    # 3. 测试启动 workflow
    print("\n▶️  测试 3: 启动 workflow")
    print("-" * 60)
    print(f"尝试启动 workflow: {workflow_name}")
    await tester.start_workflow(workflow_name)
    await asyncio.sleep(1)  # 等待一下
    
    # 4. 再次查看状态，确认 workflow 已启动
    print("\n📊 测试 4: 再次查看 service 状态（确认 workflow 已启动）")
    print("-" * 60)
    status = await tester.get_service_status()
    if status:
        for workflow in status.get('workflows', []):
            if workflow.get('name') == workflow_name:
                status_icon = "🟢" if workflow.get('status') == 'running' else "🔴"
                print(f"  {status_icon} {workflow.get('name')}: {workflow.get('status')}")
    
    # 5. 测试上传 workflow 配置文件
    print("\n📤 测试 5: 上传 workflow 配置文件")
    print("-" * 60)
    config_path = Path(__file__).parent / "configs" / "workflow" / "query_tags_workflow.yaml"
    if config_path.exists():
        print(f"上传配置文件: {config_path}")
        await tester.upload_workflow_config(str(config_path))
    else:
        print(f"⚠️  配置文件不存在: {config_path}，跳过上传测试")
    
    # 6. 最终状态查看
    print("\n📊 测试 6: 最终状态查看")
    print("-" * 60)
    status = await tester.get_service_status()
    if status:
        print(f"Service 状态:")
        print(json.dumps(status, indent=2, ensure_ascii=False))
    
    print("\n" + "=" * 60)
    print("测试完成")
    print("=" * 60)


async def main():
    """主函数"""
    parser = argparse.ArgumentParser(
        description="Service Router 测试脚本",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
示例用法:
  # 运行所有测试
  python test_service_router.py
  
  # 指定服务地址
  python test_service_router.py --base-url http://localhost:37200
  
  # 只查看状态
  python test_service_router.py --status-only
  
  # 启动特定 workflow
  python test_service_router.py --start-workflow query_tags_workflow
  
  # 停止特定 workflow
  python test_service_router.py --stop-workflow query_tags_workflow
  
  # 上传配置文件
  python test_service_router.py --upload-config configs/workflow/query_tags_workflow.yaml
        """
    )
    parser.add_argument(
        "--base-url",
        type=str,
        default="http://localhost:37200",
        help="Service 基础 URL (默认: http://localhost:37200)"
    )
    parser.add_argument(
        "--status-only",
        action="store_true",
        help="只查看 service 状态"
    )
    parser.add_argument(
        "--start-workflow",
        type=str,
        help="启动指定的 workflow"
    )
    parser.add_argument(
        "--stop-workflow",
        type=str,
        help="停止指定的 workflow"
    )
    parser.add_argument(
        "--upload-config",
        type=str,
        help="上传并加载 workflow 配置文件"
    )
    parser.add_argument(
        "--workflow-name",
        type=str,
        help="上传配置文件时指定的 workflow 名称（可选）"
    )
    
    args = parser.parse_args()
    
    tester = ServiceRouterTester(base_url=args.base_url)
    
    try:
        if args.status_only:
            # 只查看状态
            status = await tester.get_service_status()
            if status:
                print(json.dumps(status, indent=2, ensure_ascii=False))
        elif args.start_workflow:
            # 启动 workflow
            await tester.start_workflow(args.start_workflow)
        elif args.stop_workflow:
            # 停止 workflow
            await tester.stop_workflow(args.stop_workflow)
        elif args.upload_config:
            # 上传配置文件
            await tester.upload_workflow_config(args.upload_config, args.workflow_name)
        else:
            # 运行所有测试
            await test_all(tester)
    except KeyboardInterrupt:
        print("\n\n⚠️  测试被用户中断")
        sys.exit(1)
    except Exception as e:
        print(f"\n❌ 发生未预期的错误: {e}")
        import traceback
        traceback.print_exc()
        sys.exit(1)


if __name__ == "__main__":
    asyncio.run(main())




from __future__ import annotations

import os
import asyncio
import threading
import uuid
from importlib.metadata import version
from loguru import logger
from typing import Callable, AsyncIterator, Awaitable, Any, TYPE_CHECKING
from service_forge.workflow.node import node_register
from service_forge.workflow.workflow_factory import create_workflow_group
from service_forge.api.http_api import start_fastapi_server
from service_forge.api.kafka_api import start_kafka_server
from service_forge.db.database import DatabaseManager
from service_forge.api.http_api_doc import generate_service_http_api_doc
from service_forge.sft.config.sf_metadata import SfMetadata
from service_forge.service_config import ServiceConfig
from service_forge.current_service import set_service

if TYPE_CHECKING:
    from service_forge.workflow.workflow_group import WorkflowGroup
    from service_forge.workflow.workflow import Workflow

class Service:
    def __init__(
        self,
        metadata: SfMetadata,
        config: ServiceConfig,
        config_path: str,
        service_env: dict[str, Any] = None,
        database_manager: DatabaseManager = None,
        _handle_stream_output: Callable[[str, AsyncIterator[str]], Awaitable[None]] = None,
        _handle_query_user: Callable[[str, str], Awaitable[str]] = None,
    ) -> None:
        self.metadata = metadata
        self.config = config
        self.config_path = config_path
        self._handle_stream_output = _handle_stream_output
        self._handle_query_user = _handle_query_user
        self.service_env = {} if service_env is None else service_env
        self.database_manager = database_manager
        self.workflow_groups: list[WorkflowGroup] = []
        self.workflow_tasks: dict[str, asyncio.Task] = {}  # workflow_name -> task mapping
        self.workflow_config_map: dict[uuid.UUID, str] = {}  # workflow_id -> config_path mapping
        self.fastapi_thread: threading.Thread | None = None
        self.fastapi_loop: asyncio.AbstractEventLoop | None = None

    @property
    def name(self) -> str:
        return self.metadata.name
    
    @property
    def version(self) -> str:
        return self.metadata.version
    
    @property
    def description(self) -> str:
        return self.metadata.description

    async def start(self):
        set_service(self)
        
        if self.config.enable_http:
            fastapi_task = asyncio.create_task(start_fastapi_server(self.config.http_host, self.config.http_port))
            doc_task = asyncio.create_task(generate_service_http_api_doc(self))
        else:
            fastapi_task = None
            doc_task = None

        if self.config.enable_kafka:
            kafka_task = asyncio.create_task(start_kafka_server(f"{self.config.kafka_host}:{self.config.kafka_port}"))
        else:
            kafka_task = None

        workflow_tasks: list[asyncio.Task] = []

        for workflow_config_path in self.config.workflows:
            logger.info(f"Loading workflow from {self.parse_workflow_path(workflow_config_path)}")
            workflow_group = create_workflow_group(
                config_path=self.parse_workflow_path(workflow_config_path),
                service_env=self.service_env,
                _handle_stream_output=self._handle_stream_output, 
                _handle_query_user=self._handle_query_user,
                database_manager=self.database_manager,
            )
            self.workflow_groups.append(workflow_group)
            main_workflow = workflow_group.get_main_workflow()
            task = asyncio.create_task(workflow_group.run())
            workflow_tasks.append(task)
            self.workflow_tasks[main_workflow.id] = task
            self.workflow_config_map[main_workflow.name] = workflow_config_path

        try:
            core_tasks = []
            if fastapi_task:
                core_tasks.append(fastapi_task)
            if doc_task:
                core_tasks.append(doc_task)
            if kafka_task:
                core_tasks.append(kafka_task)
            
            all_tasks = core_tasks + workflow_tasks
            results = await asyncio.gather(*all_tasks, return_exceptions=True)
            
            # Check core tasks
            for i, result in enumerate(results[:len(core_tasks)]):
                if isinstance(result, Exception):
                    logger.error(f"Error in service {self.name} core task {i}: {result}")
                    raise result
            
            # Check workflow tasks
            for i, result in enumerate(results[len(core_tasks):], start=len(core_tasks)):
                if isinstance(result, Exception) and not isinstance(result, asyncio.CancelledError):
                    # Workflow task exception should not stop the service
                    logger.error(f"Error in service {self.name} workflow task {i}: {result}")

        except Exception as e:
            logger.error(f"Error in service {self.name}: {e}")
            if fastapi_task:
                fastapi_task.cancel()
            if kafka_task:
                kafka_task.cancel()
            for workflow_task in workflow_tasks:
                workflow_task.cancel()
            raise

    def parse_workflow_path(self, workflow_config_path: str) -> str:
        if os.path.isabs(workflow_config_path):
            return workflow_config_path
        else:
            return os.path.join(os.path.dirname(self.config_path), workflow_config_path)
    
    def get_workflow_group_by_name(self, workflow_name: str, workflow_version: str, allow_none: bool = True) -> WorkflowGroup | None:
        for workflow_group in self.workflow_groups:
            if workflow_group.get_workflow_by_name(workflow_name, workflow_version) is not None:
                return workflow_group
        if not allow_none:
            raise ValueError(f"Workflow group with name {workflow_name} and version {workflow_version} not found in service {self.name}")
        return None

    def get_workflow_group_by_id(self, workflow_id: str, allow_none: bool = True) -> WorkflowGroup | None:
        for workflow_group in self.workflow_groups:
            if workflow_group.get_workflow_by_id(workflow_id) is not None:
                return workflow_group
        if not allow_none:
            raise ValueError(f"Workflow group with id {workflow_id} not found in service {self.name}")
        return None

    def trigger_workflow(self, workflow_group: WorkflowGroup, trigger_name: str, **kwargs) -> uuid.UUID:
        workflow = workflow_group.get_main_workflow(allow_none=False)
        return workflow.trigger(trigger_name, **kwargs)

    def trigger_workflow_by_name(self, workflow_name: str, workflow_version: str, trigger_name: str, **kwargs) -> uuid.UUID:
        workflow_group = self.get_workflow_group_by_name(workflow_name, workflow_version, allow_none=False)
        return self.trigger_workflow(workflow_group, trigger_name, **kwargs)

    def trigger_workflow_by_id(self, workflow_id: str, trigger_name: str, **kwargs) -> uuid.UUID:
        workflow_group = self.get_workflow_group_by_id(workflow_id, allow_none=False)
        return self.trigger_workflow(workflow_group, trigger_name, **kwargs)

    def start_workflow(self, workflow_group: WorkflowGroup) -> bool:
        workflow = workflow_group.get_main_workflow(allow_none=False)
        if workflow.id in self.workflow_tasks:
            task = self.workflow_tasks[workflow.id]
            if not task.done():
                logger.warning(f"Workflow {workflow.id} is already running")
                return False
            del self.workflow_tasks[workflow.id]
        
        task = asyncio.create_task(workflow_group.run())
        self.workflow_tasks[workflow.id] = task
        logger.info(f"Started workflow {workflow.id}")
        return True

    def start_workflow_by_name(self, workflow_name: str, workflow_version: str) -> bool:
        workflow_group = self.get_workflow_group_by_name(workflow_name, workflow_version, allow_none=False)
        return self.start_workflow(workflow_group)
    
    def start_workflow_by_id(self, workflow_id: str) -> bool:
        workflow_group = self.get_workflow_group_by_id(workflow_id, allow_none=False)
        return self.start_workflow(workflow_group)

    async def stop_workflow(self, workflow_group: WorkflowGroup) -> bool:
        workflow = workflow_group.get_main_workflow(allow_none=False)
        if workflow.id not in self.workflow_tasks:
            logger.warning(f"Workflow {workflow.id} is not running")
            return False
        task = self.workflow_tasks[workflow.id]
        if task.done():
            logger.warning(f"Workflow {workflow.id} is already stopped")
            del self.workflow_tasks[workflow.id]
            return False
        task.cancel()
        await workflow.stop()
        try:
            await task
        except asyncio.CancelledError:
            pass
        del self.workflow_tasks[workflow.id]
        logger.info(f"Stopped workflow {workflow.id}")
        return True

    async def stop_workflow_by_name(self, workflow_name: str, workflow_version: str) -> bool:
        workflow_group = self.get_workflow_group_by_name(workflow_name, workflow_version, allow_none=False)
        return await self.stop_workflow(workflow_group)
    
    async def stop_workflow_by_id(self, workflow_id: str) -> bool:
        workflow_group = self.get_workflow_group_by_id(workflow_id, allow_none=False)
        return await self.stop_workflow(workflow_group)
    
    async def load_workflow_from_config(self, config_path: str = None, config: dict = None) -> uuid.UUID:
        workflow_group = create_workflow_group(
            config_path=config_path,
            config=config,
            service_env=self.service_env,
            _handle_stream_output=self._handle_stream_output,
            _handle_query_user=self._handle_query_user,
            database_manager=self.database_manager,
        )

        for workflow in workflow_group.workflows:
            existing_workflow_group = self.get_workflow_group_by_name(workflow.name, workflow.version)
            if existing_workflow_group is not None:
                raise ValueError(f"Workflow group with name {workflow.name} and version {workflow.version} already exists")

        self.workflow_groups.append(workflow_group)
        main_workflow = workflow_group.get_main_workflow()
        
        if main_workflow.id in self.workflow_tasks:
            await self.stop_workflow(workflow_group)

        self.start_workflow(workflow_group)
        return main_workflow.id
    
    def get_service_status(self) -> dict[str, Any]:
        workflow_statuses = []
        for workflow_group in self.workflow_groups:
            for workflow in workflow_group.workflows:
                workflow_id = workflow.id
                workflow_version = workflow.version
                workflow_config = workflow.config
                workflow_name = workflow.name
                is_running = workflow_id in self.workflow_tasks and not self.workflow_tasks[workflow_id].done()
                config_path = self.workflow_config_map.get(workflow_name, "unknown")
                workflow_statuses.append({
                    "name": workflow_name,
                    "id": workflow_id,
                    "version": workflow_version,
                    "config": workflow_config,
                    "description": workflow.description,
                    "status": "running" if is_running else "stopped",
                    "config_path": config_path,
                })
        
        return {
            "name": self.name,
            "version": self.version,
            "description": self.description,
            "workflows": workflow_statuses,
        }
    
    @staticmethod
    def from_config(metadata: SfMetadata, service_env: dict[str, Any] = None, config: ServiceConfig = None) -> Service:
        try:
            service_forge_version = version("service-forge")
            logger.info(f"service-forge version: {service_forge_version}")
        except Exception as e:
            logger.warning(f"Failed to get service-forge version: {e}")
        
        if config is not None:
            config_path = None
        else:
            config_path = metadata.service_config
            config = ServiceConfig.from_yaml_file(config_path)
        database_manager = DatabaseManager.from_config(config=config)
        return Service(
            metadata=metadata,
            config_path=config_path,
            config=config,
            service_env=service_env,
            database_manager=database_manager,
            _handle_stream_output=None,
            _handle_query_user=None,
        )

def create_service(config_path: str, name: str, version: str, service_env: dict[str, Any] = None) -> Service:
    return Service.from_config(config_path, name, version, service_env)



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ChatMessage(_message.Message):
    __slots__ = ("proto_id", "id", "content", "role", "session_id", "user_id", "is_stream", "is_stream_done")
    PROTO_ID_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTENT_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    SESSION_ID_FIELD_NUMBER: _ClassVar[int]
    USER_ID_FIELD_NUMBER: _ClassVar[int]
    IS_STREAM_FIELD_NUMBER: _ClassVar[int]
    IS_STREAM_DONE_FIELD_NUMBER: _ClassVar[int]
    proto_id: str
    id: int
    content: str
    role: str
    session_id: int
    user_id: int
    is_stream: bool
    is_stream_done: bool
    def __init__(self, proto_id: _Optional[str] = ..., id: _Optional[int] = ..., content: _Optional[str] = ..., role: _Optional[str] = ..., session_id: _Optional[int] = ..., user_id: _Optional[int] = ..., is_stream: bool = ..., is_stream_done: bool = ...) -> None: ...

class ChatHistory(_message.Message):
    __slots__ = ("proto_id", "messages")
    PROTO_ID_FIELD_NUMBER: _ClassVar[int]
    MESSAGES_FIELD_NUMBER: _ClassVar[int]
    proto_id: str
    messages: _containers.RepeatedCompositeFieldContainer[ChatMessage]
    def __init__(self, proto_id: _Optional[str] = ..., messages: _Optional[_Iterable[_Union[ChatMessage, _Mapping]]] = ...) -> None: ...



import os
import json
import requests
from pathlib import Path
from service_forge.sft.util.logger import log_error, log_info, log_success, log_warning
from service_forge.sft.config.sft_config import sft_config

def remote_deploy_tar(filename: str, service_center_url: str = None) -> None:
    """
    Remote deploy specified tar package from service-center
    """
    # If URL is not provided, try to get it from configuration
    if not service_center_url:
        service_center_url = getattr(sft_config, 'service_center_address', 'http://localhost:5000')
    
    # Ensure URL ends with /
    if not service_center_url.endswith('/'):
        service_center_url += '/'
    
    api_url = f"{service_center_url}api/v1/services/deploy-from-tar"
    
    log_info(f"Sending deployment request to {api_url}...")
    log_info(f"Tar package to deploy: {filename}")
    
    try:
        # Prepare request data
        data = {
            "filename": filename
        }
        
        # Send POST request
        response = requests.post(
            api_url,
            json=data,
            headers={'Content-Type': 'application/json'},
            timeout=300  # 5 minute timeout
        )
        
        if response.status_code != 200:
            log_error(f"Deployment request failed, status code: {response.status_code}")
            try:
                error_data = response.json()
                log_error(f"Error message: {error_data.get('message', 'Unknown error')}")
                if 'data' in error_data and error_data['data']:
                    log_error(f"Details: {json.dumps(error_data['data'], indent=2, ensure_ascii=False)}")
            except:
                log_error(f"Response content: {response.text}")
            return
        
        # Parse response data
        result = response.json()
        
        if result.get('code') != 200:
            log_error(f"Deployment failed: {result.get('message', 'Unknown error')}")
            if 'data' in result and result['data']:
                log_error(f"Details: {json.dumps(result['data'], indent=2, ensure_ascii=False)}")
            return
        
        # Deployment successful
        data = result.get('data', {})
        service_name = data.get('service_name', 'Unknown')
        version = data.get('version', 'Unknown')
        deploy_output = data.get('deploy_output', '')
        
        log_success(f"Successfully deployed service: {service_name} version: {version}")
        
        if deploy_output:
            log_info("Deployment output:")
            print(deploy_output)
        
    except requests.exceptions.Timeout:
        log_error("Deployment request timed out (exceeded 5 minutes), please check service status or try again later")
    except requests.exceptions.RequestException as e:
        log_error(f"Request failed: {str(e)}")
        log_info(f"Please check if service-center service is running normally and if the URL is correct: {service_center_url}")
    except Exception as e:
        log_error(f"Exception occurred while deploying tar package: {str(e)}")

def remote_list_and_deploy(service_center_url: str = None) -> None:
    """
    List remote tar packages first, then let user select which package to deploy
    """
    # If URL is not provided, try to get it from configuration
    if not service_center_url:
        service_center_url = getattr(sft_config, 'service_center_address', 'http://localhost:5000')
    
    # Ensure URL ends with /
    if not service_center_url.endswith('/'):
        service_center_url += '/'
    
    api_url = f"{service_center_url}api/v1/services/tar-list"
    
    log_info(f"Getting tar package list from {api_url}...")
    
    try:
        # 发送GET请求获取tar包列表
        response = requests.get(api_url, timeout=30)
        
        if response.status_code != 200:
            log_error(f"Failed to get tar package list, status code: {response.status_code}")
            return
        
        # Parse response data
        result = response.json()
        
        if result.get('code') != 200:
            log_error(f"Failed to get tar package list: {result.get('message', 'Unknown error')}")
            return
        
        tar_files = result.get('data', [])
        
        if not tar_files:
            log_info("No tar packages found")
            return
        
        # Display tar package list
        log_info("Available tar package list:")
        for i, tar_file in enumerate(tar_files, 1):
            filename = tar_file.get('filename', '-')
            service_name = tar_file.get('service_name', '-')
            version = tar_file.get('version', '-')
            deployed_status = "Deployed" if tar_file.get('deployed_status', False) else "Not Deployed"
            
            print(f"{i}. {filename} (service: {service_name}, version: {version}, status: {deployed_status})")
        
        # Let user choose
        try:
            choice = input("\nEnter the number of the tar package to deploy (enter 'q' to exit): ").strip()
            
            if choice.lower() == 'q':
                log_info("Deployment cancelled")
                return
            
            index = int(choice) - 1
            if 0 <= index < len(tar_files):
                selected_tar = tar_files[index]
                filename = selected_tar.get('filename')
                
                if selected_tar.get('deployed_status', False):
                    log_warning(f"Tar package {filename} is already deployed, continue deployment?")
                    confirm = input("Enter 'y' to continue, any other key to cancel: ").strip().lower()
                    if confirm != 'y':
                        log_info("Deployment cancelled")
                        return
                
                log_info(f"Selected for deployment: {filename}")
                remote_deploy_tar(filename, service_center_url)
            else:
                log_error("Invalid selection")
                
        except ValueError:
            log_error("Please enter a valid number")
        except KeyboardInterrupt:
            log_info("\nDeployment cancelled")
        
    except requests.exceptions.RequestException as e:
        log_error(f"Request failed: {str(e)}")
        log_info(f"Please check if service-center service is running normally and if the URL is correct: {service_center_url}")
    except Exception as e:
        log_error(f"Exception occurred while getting tar package list: {str(e)}")


import random
from openai import OpenAI
from openai import AzureOpenAI
from typing import Iterator

class LLM():
    def __init__(self, api_key: str, base_url: str, timeout: int, api_version: str | None = None):
        if api_version is not None:
            self.client = AzureOpenAI(
                api_key=api_key,
                azure_endpoint=base_url,
                timeout=timeout,
                api_version=api_version,
            )
        else:
            self.client = OpenAI(
                api_key=api_key,
                base_url=base_url,
                timeout=timeout,
            )

    def chat(self, input: str, system_prompt: str, model: str, temperature: float) -> str:
        if model.startswith("azure"):
            model = model.replace("azure-", "")

        response = self.client.chat.completions.create(
            model=model,
            messages=[
                {"role": "system", "content": system_prompt},
                {"role": "user", "content": input},
            ],
            temperature=temperature,
        )

        if response.choices[0].message.content is None:
            return "Error"
        else:
            return response.choices[0].message.content

    def chat_stream(self, input: str, system_prompt: str, model: str, temperature: float) -> Iterator[str]:
        if model.startswith("azure"):
            model = model.replace("azure-", "")

        stream = self.client.chat.completions.create(
            model=model,
            messages=[
                {"role": "system", "content": system_prompt},
                {"role": "user", "content": input},
            ],
            temperature=temperature,
            stream=True,
        )

        for chunk in stream:
            if chunk.choices[0].delta.content is not None:
                yield chunk.choices[0].delta.content



from typing import List, Optional
import typer
from rich.console import Console
from rich.table import Table
from kubernetes import client, config
from kubernetes.client.rest import ApiException

from service_forge.sft.config.sft_config import sft_config
from service_forge.sft.util.logger import log_error, log_info, log_success, log_warning
from service_forge.sft.kubernetes.kubernetes_manager import KubernetesManager

def list_services() -> None:
    namespace = sft_config.k8s_namespace
    kubernetes_manager = KubernetesManager()
    services = kubernetes_manager.get_services_in_namespace(namespace)
    
    if not services:
        log_warning(f"No services starting with 'sf-' found in namespace '{namespace}'")
        return
    
    console = Console()
    table = Table(title=f"Services in namespace '{namespace}' (sf-*)", show_header=True, header_style="bold magenta")
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Type", style="green")
    table.add_column("Port", style="yellow")
    table.add_column("Target Port", style="yellow")
    
    for service_name in sorted(services):
        details = kubernetes_manager.get_service_details(namespace, service_name)
        table.add_row(
            details.name,
            details.type or "-",
            str(details.port or "-"),
            str(details.target_port or "-")
        )
    
    console.print(table)
    log_info(f"Found {len(services)} service(s)")

def show_logs(
    namespace: str,
    pod_name: str,
    container_name: str,
    tail: int,
    follow: bool,
    previous: bool
) -> None:
    kubernetes_manager = KubernetesManager()
    try:
        if not follow:
            log_info(f"Fetching logs from pod '{pod_name}' (container: {container_name})...")
        
        logs = kubernetes_manager.get_pod_logs(
            namespace=namespace,
            pod_name=pod_name,
            container_name=container_name,
            tail=tail,
            follow=follow,
            previous=previous
        )
        
        if follow:
            try:
                for line in logs:
                    log_info(line, end="")
            except KeyboardInterrupt:
                log_warning("Log streaming interrupted")
                raise typer.Exit(0)
        else:
            if logs:
                log_info(logs)
            else:
                log_warning(f"No logs available for pod '{pod_name}' container '{container_name}'")
                
    except ApiException as e:
        if e.status == 404:
            log_error(f"Pod '{pod_name}' or container '{container_name}' not found")
        elif e.status == 400:
            log_error(f"Bad request: {e.reason}")
            if "previous" in str(e.body).lower():
                log_info("Note: 'previous' flag only works for stopped containers")
        else:
            log_error(f"Failed to get logs: {e.reason}")
            if e.body:
                log_error(f"Error details: {e.body}")
    except Exception as e:
        log_error(f"Failed to get logs: {e}")


def show_service_logs(
    service_name: str,
    container: Optional[str] = None,
    tail: int = 100,
    follow: bool = False,
    previous: bool = False
) -> None:
    namespace = sft_config.k8s_namespace
    kubernetes_manager = KubernetesManager()
    
    if not service_name.startswith("sf-"):
        log_warning(f"Service name '{service_name}' does not start with 'sf-'. Proceeding anyway...")
    
    services = kubernetes_manager.get_services_in_namespace(namespace)
    if service_name not in services:
        log_error(f"Service '{service_name}' not found in namespace '{namespace}'")
        log_info(f"Available services: {', '.join(services) if services else 'None'}")
        raise typer.Exit(1)
    
    pod_names = kubernetes_manager.get_pods_for_service(namespace, service_name)
    if not pod_names:
        log_error(f"No pods found for service '{service_name}' in namespace '{namespace}'")
        raise typer.Exit(1)
    
    log_info(f"Found {len(pod_names)} pod(s) for service '{service_name}'")
    
    for pod_name in pod_names:
        containers = kubernetes_manager.get_pod_containers(namespace, pod_name)
        
        if not containers:
            log_warning(f"No containers found in pod '{pod_name}'")
            continue

        if container and container not in containers:
            log_error(f"Container '{container}' not found in pod '{pod_name}'")
            log_info(f"Available containers: {', '.join(containers)}")
            continue
        
        target_containers = [container] if container else containers
        
        for container_name in target_containers:
            show_logs(namespace, pod_name, container_name, tail, follow, previous)

def delete_service(service_name: str, force: bool = False) -> None:
    namespace = sft_config.k8s_namespace
    
    if not service_name.startswith("sf-"):
        log_error(f"Service name '{service_name}' does not start with 'sf-'")
        raise typer.Exit(1)
    
    kubernetes_manager = KubernetesManager()

    services = kubernetes_manager.get_services_in_namespace(namespace)
    if service_name not in services:
        log_warning(f"Service '{service_name}' not found in namespace '{namespace}'")
        log_info(f"Available services: {', '.join(services) if services else 'None'}")
    
    log_info(f"Deleting service '{service_name}' from namespace '{namespace}'...")
    kubernetes_manager.delete_service(namespace, service_name, force)
    


import asyncio
from typing import TYPE_CHECKING, Any, Union
from service_forge.api.http_api import fastapi_app
from fastapi.openapi.utils import get_openapi
from pydantic import BaseModel
from google.protobuf.message import Message
from google.protobuf import descriptor as _descriptor
from a2a.utils.constants import AGENT_CARD_WELL_KNOWN_PATH, EXTENDED_AGENT_CARD_PATH

if TYPE_CHECKING:
    from service_forge.service import Service
    from service_forge.workflow.workflow import Workflow
    from service_forge.workflow.workflow_group import WorkflowGroup

def _protobuf_type_to_json_schema_type(field_type: int) -> dict[str, Any]:
    """将 protobuf 字段类型转换为 JSON schema 类型"""
    type_map = {
        _descriptor.FieldDescriptor.TYPE_DOUBLE: {"type": "number", "format": "double"},
        _descriptor.FieldDescriptor.TYPE_FLOAT: {"type": "number", "format": "float"},
        _descriptor.FieldDescriptor.TYPE_INT64: {"type": "integer", "format": "int64"},
        _descriptor.FieldDescriptor.TYPE_UINT64: {"type": "integer", "format": "uint64"},
        _descriptor.FieldDescriptor.TYPE_INT32: {"type": "integer", "format": "int32"},
        _descriptor.FieldDescriptor.TYPE_UINT32: {"type": "integer", "format": "uint32"},
        _descriptor.FieldDescriptor.TYPE_FIXED64: {"type": "integer", "format": "int64"},
        _descriptor.FieldDescriptor.TYPE_FIXED32: {"type": "integer", "format": "int32"},
        _descriptor.FieldDescriptor.TYPE_SFIXED32: {"type": "integer", "format": "int32"},
        _descriptor.FieldDescriptor.TYPE_SFIXED64: {"type": "integer", "format": "int64"},
        _descriptor.FieldDescriptor.TYPE_SINT32: {"type": "integer", "format": "int32"},
        _descriptor.FieldDescriptor.TYPE_SINT64: {"type": "integer", "format": "int64"},
        _descriptor.FieldDescriptor.TYPE_BOOL: {"type": "boolean"},
        _descriptor.FieldDescriptor.TYPE_STRING: {"type": "string"},
        _descriptor.FieldDescriptor.TYPE_BYTES: {"type": "string", "format": "byte"},
    }
    return type_map.get(field_type, {"type": "string"})


def _get_protobuf_message_class_from_descriptor(message_descriptor: _descriptor.Descriptor) -> type[Message] | None:
    """从 Descriptor 获取对应的 Python Message 类"""
    try:
        from google.protobuf import symbol_database
        _sym_db = symbol_database.Default()
        return _sym_db.GetPrototype(message_descriptor)
    except Exception:
        # 如果无法通过 symbol_database 获取，尝试通过模块查找
        try:
            # 尝试从包含的文件描述符中获取
            file_desc = message_descriptor.file
            package = file_desc.package
            message_name = message_descriptor.name
            
            # 尝试导入模块（这需要知道模块路径）
            # 这里我们返回 None，让调用者处理
            return None
        except Exception:
            return None


def _protobuf_message_to_json_schema(message_class: type[Message], openapi_schema: dict[str, Any], visited: set[str] | None = None) -> dict[str, Any]:
    """将 protobuf Message 类型转换为 JSON schema"""
    if visited is None:
        visited = set()
    
    descriptor = message_class.DESCRIPTOR
    model_name = descriptor.name
    
    # 防止循环引用
    if model_name in visited:
        return {"$ref": f"#/components/schemas/{model_name}"}
    
    visited.add(model_name)
    
    schema: dict[str, Any] = {
        "type": "object",
        "properties": {},
        "required": []
    }
    
    for field in descriptor.fields:
        field_name = field.name
        field_schema: dict[str, Any] = {}
        
        if field.label == field.LABEL_REPEATED:
            # 数组类型
            if field.type == field.TYPE_MESSAGE:
                # 嵌套消息数组
                nested_message_class = _get_protobuf_message_class_from_descriptor(field.message_type)
                if nested_message_class:
                    # 确保嵌套消息的 schema 也被添加
                    _get_protobuf_schema_ref(nested_message_class, openapi_schema, visited)
                    nested_schema_ref = f"#/components/schemas/{field.message_type.name}"
                    field_schema = {
                        "type": "array",
                        "items": {"$ref": nested_schema_ref}
                    }
                else:
                    # 如果无法获取类，使用通用对象类型
                    field_schema = {
                        "type": "array",
                        "items": {"type": "object"}
                    }
            else:
                # 基本类型数组
                item_schema = _protobuf_type_to_json_schema_type(field.type)
                field_schema = {
                    "type": "array",
                    "items": item_schema
                }
        elif field.type == field.TYPE_MESSAGE:
            # 嵌套消息
            nested_message_class = _get_protobuf_message_class_from_descriptor(field.message_type)
            if nested_message_class:
                # 确保嵌套消息的 schema 也被添加
                _get_protobuf_schema_ref(nested_message_class, openapi_schema, visited)
                nested_schema_ref = f"#/components/schemas/{field.message_type.name}"
                field_schema = {"$ref": nested_schema_ref}
            else:
                # 如果无法获取类，使用通用对象类型
                field_schema = {"type": "object"}
        else:
            # 基本类型
            field_schema = _protobuf_type_to_json_schema_type(field.type)
        
        schema["properties"][field_name] = field_schema
    
    visited.remove(model_name)
    return schema


def _get_protobuf_schema_ref(message_class: type[Message], openapi_schema: dict[str, Any], visited: set[str] | None = None) -> str:
    """获取 protobuf Message 的 schema 引用"""
    model_name = message_class.DESCRIPTOR.name
    
    if "components" not in openapi_schema:
        openapi_schema["components"] = {}
    if "schemas" not in openapi_schema["components"]:
        openapi_schema["components"]["schemas"] = {}
    
    if model_name not in openapi_schema["components"]["schemas"]:
        json_schema = _protobuf_message_to_json_schema(message_class, openapi_schema, visited)
        openapi_schema["components"]["schemas"][model_name] = json_schema
    
    return f"#/components/schemas/{model_name}"


def _process_pydantic_schema_with_defs(json_schema: dict[str, Any], openapi_schema: dict[str, Any], visited: set[str] | None = None) -> dict[str, Any]:
    """处理 Pydantic 生成的 JSON schema，将 $defs 中的嵌套模型迁移到 components/schemas"""
    if visited is None:
        visited = set()
    
    # 确保 components/schemas 存在
    if "components" not in openapi_schema:
        openapi_schema["components"] = {}
    if "schemas" not in openapi_schema["components"]:
        openapi_schema["components"]["schemas"] = {}
    
    # 处理 $defs 中的嵌套模型定义
    if "$defs" in json_schema:
        defs = json_schema.pop("$defs")
        for def_name, def_schema in defs.items():
            if def_name not in visited:
                visited.add(def_name)
                # 递归处理嵌套的 $defs
                processed_def_schema = _process_pydantic_schema_with_defs(def_schema.copy(), openapi_schema, visited)
                openapi_schema["components"]["schemas"][def_name] = processed_def_schema
                visited.remove(def_name)
    
    # 更新 schema 中的 $ref 引用，从 #/$defs/ModelName 改为 #/components/schemas/ModelName
    def update_refs(obj: Any) -> Any:
        if isinstance(obj, dict):
            if "$ref" in obj:
                ref = obj["$ref"]
                if ref.startswith("#/$defs/"):
                    obj["$ref"] = ref.replace("#/$defs/", "#/components/schemas/")
            else:
                for key, value in obj.items():
                    obj[key] = update_refs(value)
        elif isinstance(obj, list):
            return [update_refs(item) for item in obj]
        return obj
    
    return update_refs(json_schema)


def _get_model_schema_ref(model: Union[type[BaseModel], type[Message]], openapi_schema: dict[str, Any], visited: set[str] | None = None) -> str:
    """获取模型的 schema 引用，支持 Pydantic BaseModel 和 protobuf Message"""
    if issubclass(model, BaseModel):
        model_name = model.__name__
        
        if "components" not in openapi_schema:
            openapi_schema["components"] = {}
        if "schemas" not in openapi_schema["components"]:
            openapi_schema["components"]["schemas"] = {}
        
        if model_name not in openapi_schema["components"]["schemas"]:
            json_schema = model.model_json_schema()
            # 处理嵌套模型定义
            processed_schema = _process_pydantic_schema_with_defs(json_schema, openapi_schema, visited)
            openapi_schema["components"]["schemas"][model_name] = processed_schema
        
        return f"#/components/schemas/{model_name}"
    elif issubclass(model, Message):
        return _get_protobuf_schema_ref(model, openapi_schema, visited)
    else:
        raise ValueError(f"Unsupported model type: {type(model)}")


def _convert_model_to_parameters(model: Union[type[BaseModel], type[Message]], openapi_schema: dict[str, Any]) -> list[dict[str, Any]]:
    """将模型转换为 OpenAPI query parameters，支持 Pydantic BaseModel 和 protobuf Message"""
    parameters = []
    
    if issubclass(model, BaseModel):
        # 先处理嵌套模型定义，确保它们都在 components/schemas 中
        # 这会返回处理后的 schema 引用，同时将所有嵌套模型添加到 components/schemas
        _get_model_schema_ref(model, openapi_schema)
        
        # 从已经处理过的 components/schemas 中获取 schema
        model_name = model.__name__
        if model_name in openapi_schema.get("components", {}).get("schemas", {}):
            processed_schema = openapi_schema["components"]["schemas"][model_name]
        else:
            # 如果还没有处理过，则处理它
            json_schema = model.model_json_schema()
            processed_schema = _process_pydantic_schema_with_defs(json_schema, openapi_schema)
        
        properties = processed_schema.get("properties", {})
        required = processed_schema.get("required", [])
        
        for field_name, field_info in properties.items():
            param = {
                "name": field_name,
                "in": "query",
                "required": field_name in required,
                "schema": field_info
            }
            parameters.append(param)
    elif issubclass(model, Message):
        descriptor = model.DESCRIPTOR
        for field in descriptor.fields:
            field_name = field.name
            field_schema = _protobuf_type_to_json_schema_type(field.type)
            
            # 对于数组和嵌套消息，在 query 参数中可能不太适用，但我们可以尝试
            if field.label == field.LABEL_REPEATED:
                if field.type == field.TYPE_MESSAGE:
                    # 嵌套消息数组，在 query 中不支持，跳过
                    continue
                else:
                    # 基本类型数组
                    field_schema = {
                        "type": "array",
                        "items": field_schema
                    }
            elif field.type == field.TYPE_MESSAGE:
                # 嵌套消息，在 query 中不支持，跳过
                continue
            
            param = {
                "name": field_name,
                "in": "query",
                "required": False,  # protobuf 字段在 proto3 中默认都是可选的
                "schema": field_schema
            }
            parameters.append(param)
    
    return parameters


def _convert_model_to_request_body(model: Union[type[BaseModel], type[Message]], openapi_schema: dict[str, Any]) -> dict[str, Any]:
    """将模型转换为 OpenAPI request body，支持 Pydantic BaseModel 和 protobuf Message"""
    schema_ref = _get_model_schema_ref(model, openapi_schema)
    return {
        "content": {
            "application/json": {
                "schema": {"$ref": schema_ref}
            }
        }
    }


def _convert_model_to_response_schema(model: Union[type[BaseModel], type[Message]], openapi_schema: dict[str, Any]) -> dict[str, Any]:
    """将模型转换为 OpenAPI response schema，支持 Pydantic BaseModel 和 protobuf Message"""
    schema_ref = _get_model_schema_ref(model, openapi_schema)
    return {
        "description": "Success",
        "content": {
            "application/json": {
                "schema": {"$ref": schema_ref}
            }
        }
    }


async def generate_service_http_api_doc(service: 'Service') -> None:
    await asyncio.sleep(1)
    openapi_schema = get_openapi(
        title=service.name,
        version=service.version,
        description=service.description,
        routes=fastapi_app.routes,
    )


    from service_forge.workflow.triggers.fast_api_trigger import FastAPITrigger
    from service_forge.workflow.triggers.a2a_api_trigger import A2AAPITrigger

    for workflow_group in service.workflow_groups:
        main_workflow = workflow_group.get_main_workflow()
        fastapi_triggers = [node for node in main_workflow.nodes if isinstance(node, FastAPITrigger)]
        a2a_triggers = [node for node in main_workflow.nodes if isinstance(node, A2AAPITrigger)]

        # TODO: multiple output ports
        if main_workflow.output_ports:
            output_type = main_workflow.output_ports[0].port.type
        else:
            output_type = None

        for trigger in fastapi_triggers:
            path = trigger.get_input_port_by_name("path").value
            method = trigger.get_input_port_by_name("method").value
            data_type = trigger.get_input_port_by_name("data_type").value

            if "paths" not in openapi_schema:
                openapi_schema["paths"] = {}
            if path not in openapi_schema["paths"]:
                openapi_schema["paths"][path] = {}
            
            operation: dict[str, Any] = {
                "summary": main_workflow.name,
                "description": main_workflow.description,
            }
            
            method_lower = method.lower()
            if method_lower == "get":
                if data_type and isinstance(data_type, type) and (issubclass(data_type, BaseModel) or issubclass(data_type, Message)):
                    operation["parameters"] = _convert_model_to_parameters(data_type, openapi_schema)
            else:
                if data_type and isinstance(data_type, type) and (issubclass(data_type, BaseModel) or issubclass(data_type, Message)):
                    operation["requestBody"] = _convert_model_to_request_body(data_type, openapi_schema)
            
            if output_type and isinstance(output_type, type) and (issubclass(output_type, BaseModel) or issubclass(output_type, Message)):
                operation["responses"] = {
                    "200": _convert_model_to_response_schema(output_type, openapi_schema)
                }
            else:
                operation["responses"] = {
                    "200": {
                        "description": "Success",
                    }
                }
            
            openapi_schema["paths"][path][method_lower] = operation

        # Handle A2A triggers
        for trigger in a2a_triggers:
            if not trigger.agent_card:
                continue
            
            agent_card = trigger.agent_card
            base_path = "/a2a"
            
            # Add agent card endpoint
            agent_card_path = base_path + AGENT_CARD_WELL_KNOWN_PATH
            if "paths" not in openapi_schema:
                openapi_schema["paths"] = {}
            if agent_card_path not in openapi_schema["paths"]:
                openapi_schema["paths"][agent_card_path] = {}
            
            openapi_schema["paths"][agent_card_path]["get"] = {
                "summary": f"Get {agent_card.name} Agent Card",
                "description": agent_card.description or f"Retrieve the agent card for {agent_card.name}",
                "tags": ["A2A Agent"],
                "responses": {
                    "200": {
                        "description": "Agent card JSON",
                        "content": {
                            "application/json": {
                                "schema": {
                                    "type": "object",
                                    "description": "A2A Agent Card specification"
                                }
                            }
                        }
                    }
                }
            }
            
            # Add JSON-RPC endpoint
            rpc_path = base_path + "/"
            if rpc_path not in openapi_schema["paths"]:
                openapi_schema["paths"][rpc_path] = {}
            
            openapi_schema["paths"][rpc_path]["post"] = {
                "summary": f"{agent_card.name} JSON-RPC Endpoint",
                "description": agent_card.description or f"JSON-RPC endpoint for {agent_card.name}",
                "tags": ["A2A Agent"],
                "requestBody": {
                    "required": True,
                    "content": {
                        "application/json": {
                            "schema": {
                                "type": "object",
                                "description": "JSON-RPC request",
                                "properties": {
                                    "jsonrpc": {"type": "string", "example": "2.0"},
                                    "method": {"type": "string"},
                                    "params": {"type": "object"},
                                    "id": {"type": ["string", "integer", "null"]}
                                },
                                "required": ["jsonrpc", "method"]
                            }
                        }
                    }
                },
                "responses": {
                    "200": {
                        "description": "JSON-RPC response",
                        "content": {
                            "application/json": {
                                "schema": {
                                    "type": "object",
                                    "description": "JSON-RPC response"
                                }
                            }
                        }
                    }
                }
            }
            
            # Add extended card endpoint if supported
            if agent_card.supports_authenticated_extended_card:
                extended_card_path = base_path + EXTENDED_AGENT_CARD_PATH
                if extended_card_path not in openapi_schema["paths"]:
                    openapi_schema["paths"][extended_card_path] = {}
                
                openapi_schema["paths"][extended_card_path]["get"] = {
                    "summary": f"Get {agent_card.name} Authenticated Extended Agent Card",
                    "description": f"Retrieve the authenticated extended agent card for {agent_card.name}",
                    "tags": ["A2A Agent"],
                    "security": [{"bearerAuth": []}],
                    "responses": {
                        "200": {
                            "description": "Extended agent card JSON",
                            "content": {
                                "application/json": {
                                    "schema": {
                                        "type": "object",
                                        "description": "A2A Extended Agent Card specification"
                                    }
                                }
                            }
                        }
                    }
                }

    fastapi_app.openapi_schema = openapi_schema



import asyncio
from typing import AsyncIterator
from service_forge.workflow.node import Node
from service_forge.workflow.port import Port
from model.http.test_sse_model import TestSSEModel

class TestContextNode(Node):
    DEFAULT_INPUT_PORTS = [
        Port("message", TestSSEModel)
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("result", TestSSEModel)
    ]

    def __init__(self, name: str):
        super().__init__(name)

    async def _run(self, message: TestSSEModel) -> AsyncIterator[str]:
        if self.context.variables.get('count') is None:
            self.context.variables['count'] = 0
        self.context.variables['count'] += 1
        print(f"TestContextNode {self.name} count: {self.context.variables['count']}")
        self.activate_output_edges('result', message)


import json
import asyncio
import argparse
from aiokafka import AIOKafkaConsumer
import google.protobuf.message
from google.protobuf import descriptor_pool
import sys
import os

# 添加项目路径以导入protobuf模块
sys.path.append(os.path.join(os.path.dirname(__file__), '..', 'example', 'tag-service', 'proto'))

def detect_and_parse_data(raw_data: bytes) -> tuple[str, dict | None]:
    """自动检测数据格式并解析，返回格式类型和解析后的数据"""
    try:
        # 首先尝试JSON格式
        body = json.loads(raw_data.decode("utf-8"))
        return "JSON", body
    except json.JSONDecodeError:
        pass
    
    # 尝试protobuf格式 - 尝试多种已知的protobuf消息类型
    protobuf_types = [
        "tagService.tag_pb2.Tag",
    ]
    
    for pb_type_str in protobuf_types:
        try:
            # 动态导入protobuf类型
            module_name, class_name = pb_type_str.rsplit('.', 1)
            module = __import__(module_name, fromlist=[class_name])
            pb_class = getattr(module, class_name)
            
            # 尝试解析
            pb_obj = pb_class()
            pb_obj.ParseFromString(raw_data)
            body = protobuf_to_dict(pb_obj)
            return f"Protobuf ({pb_type_str})", body
        except Exception:
            continue
    
    # 如果都失败了，返回原始数据
    return "Unknown", {"raw_data": raw_data.hex(), "size": len(raw_data)}

def protobuf_to_dict(pb_obj: google.protobuf.message.Message) -> dict:
    """将protobuf对象转换为字典"""
    result = {}
    for field_descriptor in pb_obj.DESCRIPTOR.fields:
        field_name = field_descriptor.name
        field_value = getattr(pb_obj, field_name)
        
        if field_descriptor.label == field_descriptor.LABEL_REPEATED:
            # 处理重复字段
            result[field_name] = list(field_value)
        elif field_descriptor.type == field_descriptor.TYPE_MESSAGE:
            # 处理嵌套消息
            if field_value:
                result[field_name] = protobuf_to_dict(field_value)
        else:
            # 处理基本类型
            result[field_name] = field_value
    return result

async def consume_messages(bootstrap_servers: str, topic: str, group_id: str = "test_group", format_mode: str = "auto"):
    # TODO: proper config
    consumer = AIOKafkaConsumer(
        topic,
        bootstrap_servers=bootstrap_servers,
        group_id=group_id,
        auto_offset_reset="earliest",
        enable_auto_commit=True,
        session_timeout_ms=300000,
        heartbeat_interval_ms=10000,
        max_poll_interval_ms=3000000,
    )
    
    await consumer.start()
    try:
        print(f"🎧 开始监听 topic: {topic}, group_id: {group_id}")
        print("按 Ctrl+C 停止消费...")
        
        async for message in consumer:
            print(f"📨 收到消息:")
            print(f"  Topic: {message.topic}")
            print(f"  Partition: {message.partition}")
            print(f"  Offset: {message.offset}")
            print(f"  Timestamp: {message.timestamp}")
            print(f"  Key: {message.key}")
            
            # 根据格式模式解析数据
            if format_mode == "json":
                try:
                    parsed_data = json.loads(message.value.decode("utf-8"))
                    data_format = "JSON"
                except json.JSONDecodeError:
                    data_format = "JSON Parse Error"
                    parsed_data = {"error": "无法解析为JSON格式"}
            elif format_mode == "protobuf":
                # 尝试protobuf格式
                data_format, parsed_data = detect_and_parse_data(message.value)
                if not data_format.startswith("Protobuf"):
                    data_format = "Protobuf Parse Error"
                    parsed_data = {"error": "无法解析为Protobuf格式"}
            else:  # auto
                data_format, parsed_data = detect_and_parse_data(message.value)
            
            print(f"  Format: {data_format}")
            print(f"  Value: {json.dumps(parsed_data, indent=2, ensure_ascii=False)}")
            print("-" * 50)
            
    except KeyboardInterrupt:
        print("\n⏹️  停止消费...")
    finally:
        await consumer.stop()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Kafka Consumer Script - 支持 JSON 和 Protobuf 格式")
    parser.add_argument("--topic", type=str, default="test_topic", help="要消费的 topic 名称")
    parser.add_argument("--group-id", type=str, default="test_group", help="消费者组 ID")
    parser.add_argument("--bootstrap-servers", type=str, default="localhost:9092", help="Kafka 服务器地址")
    parser.add_argument("--format", type=str, choices=["auto", "json", "protobuf"], default="auto", 
                       help="数据格式 (auto: 自动检测, json: 仅JSON, protobuf: 仅Protobuf)")
    args = parser.parse_args()

    print(f"🚀 启动 Kafka Consumer")
    print(f"  Topic: {args.topic}")
    print(f"  Group ID: {args.group_id}")
    print(f"  Bootstrap Servers: {args.bootstrap_servers}")
    print(f"  Format: {args.format}")
    print()

    asyncio.run(consume_messages(args.bootstrap_servers, args.topic, args.group_id, args.format))



from __future__ import annotations
import asyncio
from typing import AsyncIterator
from service_forge.workflow.node import Node
from service_forge.workflow.port import Port

class PrintNode(Node):
    DEFAULT_INPUT_PORTS = [
        Port("TRIGGER", bool),
        Port("message", str),
    ]

    DEFAULT_OUTPUT_PORTS = [
    ]

    AUTO_FILL_INPUT_PORTS = [('TRIGGER', True)]

    def __init__(
        self, 
        name: str,
    ) -> None:
        super().__init__(
            name,
        )

    async def _run(self, message: str) -> AsyncIterator[str]:
        for char in str(message):
            await asyncio.sleep(0.1)
            yield char



import uuid
from omegaconf import OmegaConf
from typing import Callable, Awaitable, AsyncIterator, Any
from copy import deepcopy

from service_forge.workflow.workflow_callback import BuiltinWorkflowCallback
from .workflow import Workflow
from .workflow_group import WorkflowGroup, WORKFLOW_DEFAULT_VERSION
from .node import Node
from .edge import Edge
from .port import Port, parse_port_name, create_workflow_input_port, create_sub_workflow_input_port
from .node import node_register
from .nodes import *
from .triggers import *
from .context import Context
from ..db.database import DatabaseManager
from .workflow_config import WorkflowConfig, WorkflowGroupConfig

# WORKFLOW_KEY_NAME = 'name'
# WORKFLOW_KEY_DESCRIPTION = 'description'
# WORKFLOW_KEY_VERSION = 'version'
# WORKFLOW_KEY_NODES = 'nodes'
# WORKFLOW_KEY_INPUTS = 'inputs'
# WORKFLOW_KEY_OUTPUTS = 'outputs'

# NODE_KEY_NAME = 'name'
# NODE_KEY_TYPE = 'type'
# NODE_KEY_ARGS = 'args'
# NODE_KEY_OUTPUTS = 'outputs'
# # NODE_KEY_INPUT_PORTS = 'input_ports'
# # NODE_KEY_OUTPUT_PORTS = 'output_ports'
# NODE_KEY_SUB_WORKFLOWS = 'sub_workflows'
# NODE_KEY_SUB_WORKFLOWS_INPUT_PORTS = 'sub_workflows_input_ports'

# PORT_KEY_NAME = 'name'
# PORT_KEY_PORT = 'port'
# PORT_KEY_VALUE = 'value'

def parse_argument(arg: Any, service_env: dict[str, Any] = None) -> Any:
    # TODO: support import variables
    if type(arg) == str and arg.startswith(f'<{{') and arg.endswith(f'}}>'):
        key = arg[2:-2]
        if key not in service_env:
            raise ValueError(f"Key {key} not found in service env.")
        return service_env[key]
    return arg

def create_workflow(
    config_path: str = None,
    config: WorkflowConfig = None,
    service_env: dict[str, Any] = None,
    workflows: WorkflowGroup = None,
    _handle_stream_output: Callable[[str, AsyncIterator[str]], Awaitable[None]] | None = None,
    _handle_query_user: Callable[[str, str], Awaitable[str]] | None = None,
    database_manager: DatabaseManager = None,
) -> Workflow:
    if config is None:
        if config_path is not None:
            config = WorkflowConfig.from_yaml_file(config_path)
        else:
            raise ValueError("Either config_path or config must be provided")
    
    workflow = Workflow(
        id = uuid.uuid4(),
        config = config,
        nodes = [],
        input_ports = [],
        output_ports = [],
        _handle_stream_output = _handle_stream_output,
        _handle_query_user = _handle_query_user,
        database_manager = database_manager,
        # TODO: max_concurrent_runs
        callbacks = [BuiltinWorkflowCallback()],
        global_context = Context(variables={}),
    )

    nodes: dict[str, Node] = {}

    for node_config in config.nodes:
        params = {
            "name": node_config.name,
        }
        
        node: Node = node_register.instance(node_config.type, ignore_keys=['type'], kwargs=params)

        # Context
        node.context = Context(variables = {})

        # Input ports
        node.input_ports = deepcopy(node.DEFAULT_INPUT_PORTS)
        for input_port in node.input_ports:
            input_port.node = node

        # Output ports
        node.output_ports = deepcopy(node.DEFAULT_OUTPUT_PORTS)
        for output_port in node.output_ports:
            output_port.node = node
        
        # Sub workflows
        if node_config.sub_workflows is not None:
            sub_workflows: WorkflowGroup = WorkflowGroup(workflows=[])
            for sub_workflow_config in node_config.sub_workflows:
                sub_workflow = workflows.get_workflow_by_name(sub_workflow_config.name, sub_workflow_config.version)
                sub_workflows.add_workflow(deepcopy(sub_workflow))
            node.sub_workflows = sub_workflows

        # Sub workflows input ports
        if node_config.sub_workflows_input_ports is not None:
            for sub_workflow_input_port_config in node_config.sub_workflows_input_ports:
                name = sub_workflow_input_port_config.name
                sub_workflow_name, sub_workflow_port_name = parse_port_name(sub_workflow_input_port_config.port)
                sub_workflow = node.sub_workflows.get_workflow_by_name(sub_workflow_name, sub_workflow_config.version)
                if sub_workflow is None:
                    raise ValueError(f"{sub_workflow_name} is not a valid sub workflow.")
                sub_workflow_port = sub_workflow.get_input_port_by_name(sub_workflow_port_name)
                if sub_workflow_port is None:
                    raise ValueError(f"{sub_workflow_port_name} is not a valid input port.")
                value = sub_workflow_input_port_config.value
                node.input_ports.append(create_sub_workflow_input_port(name=name, node=node, port=sub_workflow_port, value=value))

        # Sub workflows output ports
        ...

        # Hooks
        if _handle_query_user is None:
            node.query_user = workflow.handle_query_user
        else:
            node.query_user = _handle_query_user

        nodes[node_config.name] = node

    # Edges
    for node_config in config.nodes:
        start_node = nodes[node_config.name]
        if node_config.outputs is not None:
            for key, value in node_config.outputs.items():
                if value is None:
                    continue

                if type(value) is str:
                    value = [value]

                for edge_value in value:
                    end_node_name, end_port_name = parse_port_name(edge_value)
                    end_node = nodes[end_node_name]

                    start_node.try_create_extended_output_port(key)
                    end_node.try_create_extended_input_port(end_port_name)

                    start_port = start_node.get_output_port_by_name(key)
                    end_port = end_node.get_input_port_by_name(end_port_name)

                    if start_port is None:
                        raise ValueError(f"{key} is not a valid output port.")
                    if end_port is None:
                        raise ValueError(f"{end_port_name} is not a valid input port.")

                    edge = Edge(start_node, end_node, start_port, end_port)

                    start_node.output_edges.append(edge)
                    end_node.input_edges.append(edge)

    workflow.add_nodes(list(nodes.values()))
    
    # Inputs
    if config.inputs is not None:
        for port_config in config.inputs:
            name = port_config.name
            node_name, node_port_name = parse_port_name(port_config.port)
            if node_name not in nodes:
                raise ValueError(f"{node_name} is not a valid node.")
            node = nodes[node_name]
            port = node.get_input_port_by_name(node_port_name)
            if port is None:
                raise ValueError(f"{node_port_name} is not a valid input port.")
            value = port_config.value
            workflow.input_ports.append(create_workflow_input_port(name=name, port=port, value=value))

    # Outputs
    if config.outputs is not None:
        for port_config in config.outputs:
            name = port_config.name
            node_name, node_port_name = parse_port_name(port_config.port)
            if node_name not in nodes:
                raise ValueError(f"{node_name} is not a valid node.")
            node = nodes[node_name]
            port = node.get_output_port_by_name(node_port_name)
            if port is None:
                raise ValueError(f"{node_port_name} is not a valid output port.")
            output_port = Port(name=name, type=Any, port=port)
            workflow.output_ports.append(output_port)
            edge = Edge(node, None, port, output_port)
            node.output_edges.append(edge)

    for node_config in config.nodes:
        node = nodes[node_config.name]
        # Arguments
        if node_config.args is not None:
            for key, value in node_config.args.items():
                node.fill_input_by_name(key, parse_argument(value, service_env=service_env))

    return workflow

def create_workflow_group(
    config_path: str = None,
    config: WorkflowConfig | WorkflowGroupConfig = None,
    service_env: dict[str, Any] = None,
    _handle_stream_output: Callable[[str, AsyncIterator[str]], Awaitable[None]] = None,
    _handle_query_user: Callable[[str, str], Awaitable[str]] = None,
    database_manager: DatabaseManager = None,
) -> WorkflowGroup:

    if config is None:
        if config_path is not None:
            try:
                config = WorkflowGroupConfig.from_yaml_file(config_path)
            except:
                config = WorkflowConfig.from_yaml_file(config_path)
        else:
            raise ValueError("Either config_path or config must be provided")

    if type(config) == WorkflowConfig:
        workflow = create_workflow(
            config_path=config_path if config_path else None,
            config=config,
            service_env=service_env,
            _handle_stream_output=_handle_stream_output,
            _handle_query_user=_handle_query_user,
            database_manager=database_manager,
        )
        return WorkflowGroup(workflows=[workflow], main_workflow_name=workflow.name, main_workflow_version=workflow.version)
    elif type(config) == WorkflowGroupConfig:
        workflows = WorkflowGroup(
            workflows=[],
            main_workflow_name=config.main_workflow_name,
            main_workflow_version=config.main_workflow_version,
        )
        for workflow_config in config.workflows:
            workflows.add_workflow(create_workflow(
                config=workflow_config,
                workflows=workflows,
                service_env=service_env,
                _handle_stream_output=_handle_stream_output,
                _handle_query_user=_handle_query_user,
                database_manager=database_manager,
            ))
        return workflows



from rich.console import Console
from typing import Any

console = Console()

def log_error(message: str, **kwargs: Any) -> None:
    console.print(f"[red]{message}[/red]", **kwargs)

def log_info(message: str, **kwargs: Any) -> None:
    console.print(f"{message}", **kwargs)

def log_success(message: str, **kwargs: Any) -> None:
    console.print(f"[green]{message}[/green]", **kwargs)

def log_warning(message: str, **kwargs: Any) -> None:
    console.print(f"[yellow]{message}[/yellow]", **kwargs)


from pydantic import BaseModel, Field
from typing import Optional, Any
from datetime import datetime
from uuid import UUID


class FeedbackCreate(BaseModel):
    """创建反馈的请求模型"""
    task_id: str = Field(..., description="工作流任务ID")
    workflow_name: str = Field(..., description="工作流名称")
    rating: Optional[int] = Field(None, ge=1, le=5, description="评分 (1-5)")
    comment: Optional[str] = Field(None, description="用户评论")
    metadata: Optional[dict[str, Any]] = Field(default_factory=dict, description="额外的元数据")


class FeedbackResponse(BaseModel):
    """反馈响应模型"""
    feedback_id: str = Field(..., description="反馈ID")
    task_id: str = Field(..., description="工作流任务ID")
    workflow_name: str = Field(..., description="工作流名称")
    rating: Optional[int] = Field(None, description="评分")
    comment: Optional[str] = Field(None, description="用户评论")
    metadata: dict[str, Any] = Field(default_factory=dict, description="元数据")
    created_at: datetime = Field(..., description="创建时间")


class FeedbackListResponse(BaseModel):
    """反馈列表响应模型"""
    total: int = Field(..., description="总数")
    feedbacks: list[FeedbackResponse] = Field(..., description="反馈列表")



from __future__ import annotations

import yaml
from pydantic import BaseModel

class ServiceFeedbackConfig(BaseModel):
    api_url: str
    api_timeout: int = 5

class ServiceDatabaseConfig(BaseModel):
    name: str
    postgres_user: str | None = None
    postgres_password: str | None = None
    postgres_host: str | None = None
    postgres_port: int | None = None
    postgres_db: str | None = None

    mongo_host: str | None = None
    mongo_port: int | None = None
    mongo_user: str | None = None
    mongo_password: str | None = None
    mongo_db: str | None = None

    redis_host: str | None = None
    redis_port: int | None = None
    redis_password: str | None = None

class ServiceConfig(BaseModel):
    name: str
    workflows: list[str]
    enable_http: bool
    http_host: str | None = None
    http_port: int | None = None
    enable_kafka: bool
    kafka_host: str | None = None
    kafka_port: int | None = None
    databases: list[ServiceDatabaseConfig] | None = None
    feedback: ServiceFeedbackConfig | None = None

    @classmethod
    def from_yaml_file(cls, filepath: str) -> ServiceConfig:
        with open(filepath, 'r', encoding='utf-8') as f:
            data = yaml.safe_load(f)
        return cls(**data)



"""
数据库迁移脚本 - 创建 feedback 表

使用方法:
1. 确保你的 service.yaml 中配置了数据库连接
2. 运行此脚本: python -m src.service_forge.db.migrations.feedback_migration

此脚本会:
- 检查 feedback 表是否存在
- 如果不存在,创建 feedback 表及索引
"""

import asyncio
import sys
from pathlib import Path

# 添加项目根目录到 Python 路径
project_root = Path(__file__).parent.parent.parent.parent
sys.path.insert(0, str(project_root))

from loguru import logger
from sqlalchemy import text
from src.service_forge.db.database import DatabaseManager
from src.service_forge.db.models.feedback import Base, FeedbackBase


async def create_database_if_not_exists(db):
    """如果数据库不存在，则创建数据库"""
    from sqlalchemy import create_engine
    from sqlalchemy.exc import OperationalError
    import asyncpg

    try:
        # 先尝试连接目标数据库
        await db.init()
        logger.info(f"✓ 数据库 '{db.postgres_db}' 已存在")
        await db.close()
        return True
    except Exception as e:
        logger.info(f"数据库 '{db.postgres_db}' 不存在，准备创建...")

        try:
            # 连接到 postgres 默认数据库
            conn = await asyncpg.connect(
                host=db.postgres_host,
                port=db.postgres_port,
                user=db.postgres_user,
                password=db.postgres_password,
                database='postgres'
            )

            # 创建数据库
            await conn.execute(f'CREATE DATABASE {db.postgres_db}')
            await conn.close()

            logger.info(f"✓ 数据库 '{db.postgres_db}' 创建成功!")
            return True

        except Exception as create_error:
            logger.error(f"创建数据库失败: {create_error}")
            return False


async def create_feedback_table(database_manager: DatabaseManager):
    """创建 feedback 表"""
    db = database_manager.get_default_postgres_database()

    if db is None:
        logger.error("未找到默认 PostgreSQL 数据库配置")
        return False

    try:
        # 1. 先确保数据库存在
        db_created = await create_database_if_not_exists(db)
        if not db_created:
            return False

        # 2. 连接到数据库
        await db.init()
        engine = db.engine

        logger.info(f"连接到数据库: {db.database_url}")

        # 3. 检查表是否存在
        async with engine.begin() as conn:
            result = await conn.execute(
                text(
                    "SELECT EXISTS (SELECT FROM information_schema.tables "
                    "WHERE table_schema = 'public' AND table_name = 'feedback');"
                )
            )
            table_exists = result.scalar()

            if table_exists:
                logger.info("✓ feedback 表已存在,跳过创建")
                return True

            logger.info("创建 feedback 表...")

            # 4. 创建表
            await conn.run_sync(Base.metadata.create_all)

            logger.info("✓ feedback 表创建成功!")
            logger.info("表结构:")
            logger.info("  - feedback_id: UUID (主键)")
            logger.info("  - task_id: VARCHAR(255) (索引)")
            logger.info("  - workflow_name: VARCHAR(255) (索引)")
            logger.info("  - rating: INTEGER (可选, 1-5)")
            logger.info("  - comment: TEXT (可选)")
            logger.info("  - metadata: JSONB (可选)")
            logger.info("  - created_at: TIMESTAMP (索引)")
            logger.info("  - updated_at: TIMESTAMP (可选)")

        await db.close()
        return True

    except Exception as e:
        logger.error(f"创建 feedback 表失败: {e}")
        return False


async def main():
    """主函数"""
    logger.info("=== Feedback 表迁移脚本 ===")

    # 提示用户提供配置文件路径
    if len(sys.argv) > 1:
        config_path = sys.argv[1]
    else:
        logger.info("请提供 service 配置文件路径:")
        logger.info("  python -m src.service_forge.db.migrations.feedback_migration <config_path>")
        logger.info("例如:")
        logger.info("  python -m src.service_forge.db.migrations.feedback_migration configs/service/my_service.yaml")
        return

    logger.info(f"读取配置文件: {config_path}")

    try:
        database_manager = DatabaseManager.from_config(config_path=config_path)
        success = await create_feedback_table(database_manager)

        if success:
            logger.info("✓ 迁移完成!")
        else:
            logger.error("✗ 迁移失败")
            sys.exit(1)

    except Exception as e:
        logger.error(f"迁移过程出错: {e}")
        sys.exit(1)


if __name__ == "__main__":
    asyncio.run(main())



import json
import asyncio
import argparse
from aiokafka import AIOKafkaProducer
import sys
import os

# 添加proto模块路径
sys.path.append(os.path.join(os.path.dirname(__file__), '..', 'example', 'tag-service', 'proto'))
from entryService import record_pb2

async def send_message(bootstrap_servers: str, topic: str, record: record_pb2.Record):
    producer = AIOKafkaProducer(
        bootstrap_servers=bootstrap_servers,
        value_serializer=lambda v: v.SerializeToString(),
    )
    await producer.start()
    try:
        await producer.send_and_wait(topic, record)
        print(f"✅ 已发送protobuf Record到 {topic}: {record}")
    finally:
        await producer.stop()

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--topic", type=str, default="tag.request")
    parser.add_argument("--text", type=str, default="我明天早上有个会议")
    args = parser.parse_args()

    # 创建Record实例
    record = record_pb2.Record(
        proto_id="123",
        id=34,
        name="example name",
        text=args.text,
        type="example type",
        user_id=58,
        group_id=10,
        file_id="example file id",
        is_recognized=True,
        is_deleted=False,
        timestamp="2021-01-01 00:00:00",
        role="user",
        is_stream=True,
        is_stream_done=False,
        force_reply=False,
        tags=[],
        status="",
    )

    asyncio.run(send_message("localhost:9092", args.topic, record))



from service_forge.workflow.node import Node
from service_forge.workflow.port import Port
from model.http.create_tag_model import CreateTagModel
from model.db.tag_base import TagBase

class HttpCreateTagNode(Node):
    DEFAULT_INPUT_PORTS = [
        Port("user_id", int),
        Port("tag", CreateTagModel)
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("tag", TagBase)
    ]

    def __init__(self, name: str):
        super().__init__(name)
    
    async def _run(self, tag: CreateTagModel, user_id: int) -> None:
        session_factory = await self.default_postgres_database.get_session_factory()
        async with session_factory() as session:
            tag = TagBase(
                name=tag.name,
                description=tag.description,
                example=tag.example,
                user_id=user_id,
                type="CUSTOM",
            )
            session.add(tag)
            await session.commit()
        self.activate_output_edges(self.get_output_port_by_name('tag'), tag)


import os
from .llm import LLM
from enum import Enum
from typing import Iterator

_llm_dicts = {}

class Model(Enum):
    GPT_4_1_NANO = "gpt-4.1-nano"
    QWEN_TURBO_LATEST = "qwen-turbo-latest"
    QWEN_PLUS_LATEST = "qwen-plus-latest"
    QWEN_MAX_LATEST = "qwen-max-latest"
    DOUBO_SEED_1_6_250615 = "doubao-seed-1-6-250615"
    DOUBO_SEED_1_6_THINKING_250615 = "doubao-seed-1-6-thinking-250615"
    DOUBO_SEED_1_6_FLASH_250615 = "doubao-seed-1-6-flash-250615"
    DEEPSEEK_V3_250324 = "deepseek-v3-250324"
    AZURE_GPT_4O_MINI = "azure-gpt-4o-mini"
    GEMINI = "gemini-2.5-flash"

    def provider(self) -> str:
        if self.value.startswith("gpt"):
            return "openai"
        elif self.value.startswith("qwen"):
            return "dashscope"
        elif self.value.startswith("doubao"):
            return "doubao"
        elif self.value.startswith("deepseek"):
            return "deepseek"
        elif self.value.startswith("azure"):
            return "azure"
        elif self.value.startswith("gemini"):
            return "gemini"
        raise ValueError(f"Invalid model: {self.value}")

def get_model(model: str) -> Model:
    if model in Model.__members__:
        return Model[model]

    model = model.upper().replace("-", "_")
    if model in Model.__members__:
        return Model[model]

    raise ValueError(f"Invalid model: {model}")
            
def get_llm(provider: str) -> LLM:
    if provider not in _llm_dicts:
        if provider == "openai":
            _llm_dicts[provider] = LLM(os.environ.get("OPENAI_API_KEY", ""), os.environ.get("OPENAI_BASE_URL", ""), int(os.environ.get("OPENAI_TIMEOUT", 2000)))
        elif provider == "doubao":
            _llm_dicts[provider] = LLM(os.environ.get("DOUBAO_API_KEY", ""), os.environ.get("DOUBAO_BASE_URL", ""), int(os.environ.get("DOUBAO_TIMEOUT", 2000)))
        elif provider == "dashscope":
            _llm_dicts[provider] = LLM(os.environ.get("DASHSCOPE_API_KEY", ""), os.environ.get("DASHSCOPE_BASE_URL", ""), int(os.environ.get("DASHSCOPE_TIMEOUT", 2000)))
        elif provider == "deepseek":
            _llm_dicts[provider] = LLM(os.environ.get("DEEPSEEK_API_KEY", ""), os.environ.get("DEEPSEEK_BASE_URL", ""), int(os.environ.get("DEEPSEEK_TIMEOUT", 2000)))
        elif provider == "azure":
            _llm_dicts[provider] = LLM(os.environ.get("AZURE_API_KEY", ""), os.environ.get("AZURE_BASE_URL", ""), int(os.environ.get("AZURE_TIMEOUT", 2000)), os.environ.get("AZURE_API_VERSION", ""))
        elif provider == "gemini":
            _llm_dicts[provider] = LLM(os.environ.get("GEMINI_API_KEY", ""), os.environ.get("GEMINI_BASE_URL", ""), int(os.environ.get("GEMINI_TIMEOUT", 2000)))
        else:
            raise ValueError(f"Invalid provider: {provider}")
    return _llm_dicts[provider]

def chat(input: str, system_prompt: str, model: Model, temperature: float) -> str:
    return get_llm(model.provider()).chat(input, system_prompt, model.value, temperature)

def chat_stream(input: str, system_prompt: str, model: Model, temperature: float) -> Iterator[str]:
    return get_llm(model.provider()).chat_stream(input, system_prompt, model.value, temperature)


#!/usr/bin/env python3
"""
创建 feedback 数据表
"""

import asyncio
import sys
from pathlib import Path

# 添加项目路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root / "src"))

from service_forge.db.database import PostgresDatabase
from service_forge.feedback.models import Base

async def create_table():
    """创建 feedback 表"""
    db = PostgresDatabase(
        name='feedback_db',
        postgres_host='localhost',
        postgres_port=5433,
        postgres_user='postgres',
        postgres_password='Luxuyang410641F',
        postgres_db='service_forge_feedback'
    )

    print("连接数据库...")
    await db.init()
    engine = db.engine

    print("创建 feedback 表...")
    async with engine.begin() as conn:
        await conn.run_sync(Base.metadata.create_all)

    await db.close()
    print('✅ 数据库表创建成功!')
    print('\n表结构:')
    print('  - feedback_id: UUID (主键)')
    print('  - task_id: VARCHAR(255) (索引)')
    print('  - workflow_name: VARCHAR(255) (索引)')
    print('  - rating: INTEGER (可选)')
    print('  - comment: TEXT (可选)')
    print('  - metadata: JSONB (可选)')
    print('  - created_at: TIMESTAMP (索引)')
    print('  - updated_at: TIMESTAMP (可选)')

if __name__ == "__main__":
    asyncio.run(create_table())



import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from routers import workflow

app = FastAPI(
    title="Service Forge API",
    description="A modern service management API",
    version="1.0.0"
)

# Configure CORS
app.add_middleware(
    CORSMiddleware,
    allow_origins=["http://localhost:5173"],  # React dev server
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Include routers
app.include_router(workflow.router, prefix="/api/v1/workflow", tags=["workflow"])

@app.get("/")
async def root():
    return {"message": "Welcome to Service Forge API"}

if __name__ == "__main__":
    uvicorn.run(
        "main:app",
        host="0.0.0.0",
        port=8001,
        reload=True,
        log_level="info"
    )



from functools import lru_cache
from pydantic_settings import BaseSettings, SettingsConfigDict


class Config(BaseSettings):
    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        case_sensitive=False,
        extra="ignore"
    )
    
    # OpenAI/LLM Configuration
    openai_base_url: str = "https://ark.cn-beijing.volces.com/api/v3"
    openai_api_key: str = ""
    openai_model: str = "deepseek-v3-250324"
    openai_timeout: int = 30
    
    # DeepSeek Configuration
    deepseek_base_url: str = "https://api.deepseek.com"
    deepseek_api_key: str = ""
    deepseek_timeout: int = 30
    
    # Doubao Configuration
    doubao_base_url: str = "https://ark.cn-beijing.volces.com/api/v3"
    doubao_api_key: str = ""
    doubao_timeout: int = 30
    
    # Dashscope Configuration
    dashscope_base_url: str = "https://dashscope.aliyuncs.com/api/v1"
    dashscope_api_key: str = ""
    dashscope_timeout: int = 30
    
    # Azure OpenAI Configuration
    azure_base_url: str = ""
    azure_api_key: str = ""
    azure_api_version: str = "2023-12-01-preview"
    azure_timeout: int = 30
    
    # LLM General Configuration
    llm_temperature: float = 0.7
    llm_max_tokens: int = 4096
    llm_default_provider: str = "deepseek"
    llm_default_model: str = "deepseek-v3-250324"
    
    # Kafka Configuration
    kafka_host: str = "localhost:9092"
    kafka_group_id: str = "kafka-pipeline-template"
    kafka_auto_offset_reset: str = "earliest"
    
    # Database Configuration
    postgres_user: str = "postgres"
    postgres_password: str = ""
    postgres_host: str = "localhost"
    postgres_port: int = 5432
    postgres_db: str = "kafka_pipeline"
    
    # HTTP Client Configuration
    http_timeout: float = 30.0
    http_max_connections: int = 100
    http_max_keepalive_connections: int = 20
    
    # Application Configuration
    app_name: str = "kafka-pipeline-template"
    app_version: str = "0.1.0"
    debug: bool = False
    log_level: str = "INFO"
    
    # API Configuration
    api_host: str = "0.0.0.0"
    api_port: int = 8000
    
    @property
    def database_url(self) -> str:
        """Generate database URL from components."""
        return f"postgresql+asyncpg://{self.postgres_user}:{self.postgres_password}@{self.postgres_host}:{self.postgres_port}/{self.postgres_db}"
    
    @property
    def database_url_sync(self) -> str:
        """Generate synchronous database URL for Alembic."""
        return f"postgresql://{self.postgres_user}:{self.postgres_password}@{self.postgres_host}:{self.postgres_port}/{self.postgres_db}"


@lru_cache()
def get_config() -> Config:
    """
    Get cached configuration instance (singleton pattern).
    
    Returns:
        Config: Application configuration instance
        
    Usage:
        from config import get_config
        
        config = get_config()
        print(config.openai_api_key)
    """
    return Config()


# Global config instance for convenience
config = get_config() 


from __future__ import annotations

from typing import Any, AsyncIterator, Union, TYPE_CHECKING, Callable, Awaitable
from abc import ABC, abstractmethod
import uuid
from loguru import logger
from .edge import Edge
from .port import Port
from .context import Context
from ..utils.register import Register
from ..db.database import DatabaseManager, PostgresDatabase, MongoDatabase, RedisDatabase
from ..utils.workflow_clone import node_clone
from .workflow_callback import CallbackEvent

if TYPE_CHECKING:
    from .workflow import Workflow

class Node(ABC):
    DEFAULT_INPUT_PORTS: list[Port] = []
    DEFAULT_OUTPUT_PORTS: list[Port] = []

    CLASS_NOT_REQUIRED_TO_REGISTER = ['Node']
    AUTO_FILL_INPUT_PORTS = []

    def __init__(
        self,
        name: str,
        context: Context = None,
        input_edges: list[Edge] = None,
        output_edges: list[Edge] = None,
        input_ports: list[Port] = DEFAULT_INPUT_PORTS,
        output_ports: list[Port] = DEFAULT_OUTPUT_PORTS,
        query_user: Callable[[str, str], Awaitable[str]] = None,
    ) -> None:
        from .workflow_group import WorkflowGroup
        self.name = name
        self.input_edges = [] if input_edges is None else input_edges
        self.output_edges = [] if output_edges is None else output_edges
        self.input_ports = input_ports
        self.output_ports = output_ports
        self.workflow: Workflow = None
        self.query_user = query_user
        self.sub_workflows: WorkflowGroup = None

        # runtime variables
        self.context = context
        self.input_variables: dict[Port, Any] = {}
        self.num_activated_input_edges = 0

    @property
    def default_postgres_database(self) -> PostgresDatabase | None:
        return self.database_manager.get_default_postgres_database()

    @property
    def default_mongo_database(self) -> MongoDatabase | None:
        return self.database_manager.get_default_mongo_database()

    @property
    def default_redis_database(self) -> RedisDatabase | None:
        return self.database_manager.get_default_redis_database()
    
    @property
    def database_manager(self) -> DatabaseManager:
        return self.workflow.database_manager

    @property
    def global_context(self) -> Context:
        return self.workflow.global_context

    def backup(self) -> None:
        # do NOT use deepcopy here
        # self.bak_context = deepcopy(self.context)
        # TODO: what if the value changes after backup?
        self.bak_input_variables = {port: value for port, value in self.input_variables.items()}
        self.bak_num_activated_input_edges = self.num_activated_input_edges

    def reset(self) -> None:
        # self.context = deepcopy(self.bak_context)
        self.input_variables = {port: value for port, value in self.bak_input_variables.items()}
        self.num_activated_input_edges = self.bak_num_activated_input_edges

    def __init_subclass__(cls) -> None:
        if cls.__name__ not in Node.CLASS_NOT_REQUIRED_TO_REGISTER:
            node_register.register(cls.__name__, cls)
        return super().__init_subclass__()

    def _query_user(self, prompt: str) -> Callable[[str, str], Awaitable[str]]:
        return self.query_user(self.name, prompt)
    
    def variables_to_params(self) -> dict[str, Any]:
        params = {port.name: self.input_variables[port] for port in self.input_variables.keys() if not port.is_extended_generated}
        for port in self.input_variables.keys():
            if port.is_extended_generated:
                if port.get_extended_name() not in params:
                    params[port.get_extended_name()] = []
                params[port.get_extended_name()].append((port.get_extended_index(), self.input_variables[port]))
                params[port.get_extended_name()].sort()
        return params

    def is_trigger(self) -> bool:
        from .trigger import Trigger
        return isinstance(self, Trigger)

    # TODO: maybe add a function before the run function?

    @abstractmethod
    async def _run(self, **kwargs) -> Union[None, AsyncIterator]:
        ...
    
    def run(self) -> Union[None, AsyncIterator]:
        for key in list(self.input_variables.keys()):
            if key and key.name[0].isupper():
                del self.input_variables[key]
        params = self.variables_to_params()
        return self._run(**params)

    def get_input_port_by_name(self, name: str) -> Port:
        # TODO: add warning if port is extended
        for port in self.input_ports:
            if port.name == name:
                return port
        return None

    def get_output_port_by_name(self, name: str) -> Port:
        # TODO: add warning if port is extended
        for port in self.output_ports:
            if port.name == name:
                return port
        return None

    def try_create_extended_input_port(self, name: str) -> None:
        for port in self.input_ports:
            if port.is_extended and name.startswith(port.name + '_') and name[len(port.name + '_'):].isdigit():
                self.input_ports.append(Port(name=name, type=port.type, node=port.node, port=port.port, value=port.value, default=port.default, is_extended=False, is_extended_generated=True))

    def try_create_extended_output_port(self, name: str) -> None:
        for port in self.output_ports:
            if port.is_extended and name.startswith(port.name + '_') and name[len(port.name + '_'):].isdigit():
                self.output_ports.append(Port(name=name, type=port.type, node=port.node, port=port.port, value=port.value, default=port.default, is_extended=False, is_extended_generated=True))

    def num_input_ports(self) -> int:
        return sum(1 for port in self.input_ports if not port.is_extended)
    
    def is_ready(self) -> bool:
        return self.num_activated_input_edges == self.num_input_ports()

    def fill_input_by_name(self, port_name: str, value: Any) -> None:
        self.try_create_extended_input_port(port_name)
        port = self.get_input_port_by_name(port_name)
        if port is None:
            raise ValueError(f'{port_name} is not a valid input port.')
        self.fill_input(port, value)
    
    def fill_input(self, port: Port, value: Any) -> None:
        port.activate(value)

    def activate_output_edges(self, port: str | Port, data: Any) -> None:
        if isinstance(port, str):
            port = self.get_output_port_by_name(port)
        for output_edge in self.output_edges:
            if output_edge.start_port == port:
                output_edge.end_port.activate(data)

    # for trigger nodes
    def prepare_output_edges(self, port: Port, data: Any) -> None:
        if isinstance(port, str):
            port = self.get_output_port_by_name(port)
        for output_edge in self.output_edges:
            if output_edge.start_port == port:
                output_edge.end_port.prepare(data)
    
    def trigger_output_edges(self, port: Port) -> None:
        if isinstance(port, str):
            port = self.get_output_port_by_name(port)
        for output_edge in self.output_edges:
            if output_edge.start_port == port:
                output_edge.end_port.trigger()

    # TODO: the result is outputed to the trigger now, maybe we should add a new function to output the result to the workflow
    def output_to_workflow(self, data: Any) -> None:
        self.workflow._handle_workflow_output(self.name, data)

    def extended_output_name(self, name: str, index: int) -> str:
        return name + '_' + str(index)

    def _clone(self, context: Context) -> Node:
        return node_clone(self, context)

    async def stream_output(self, data: Any) -> None:
         await self.workflow.call_callbacks(CallbackEvent.ON_NODE_STREAM_OUTPUT, node=self, output=data)

node_register = Register[Node]()


from __future__ import annotations
import os

from service_forge.workflow.node import Node
from service_forge.workflow.port import Port
from service_forge.llm import chat_stream, Model

class QueryLLMNode(Node):
    DEFAULT_INPUT_PORTS = [
        Port("prompt", str),
        Port("system_prompt", str),
        Port("temperature", float),
        Port("TRIGGER", bool),
    ]

    DEFAULT_OUTPUT_PORTS = [
        Port("response", str),
    ]

    AUTO_FILL_INPUT_PORTS = [('TRIGGER', True)]

    def __init__(
        self, 
        name: str,
    ) -> None:
        super().__init__(
            name,
        )

    async def _run(self, prompt: str, system_prompt: str, temperature: float) -> None:
        if os.path.exists(system_prompt):
            with open(system_prompt, "r") as f:
                system_prompt = f.read()
        if os.path.exists(prompt):
            with open(prompt, "r") as f:
                prompt = f.read()

        print(f"prompt: {prompt} temperature: {temperature}")
        response = chat_stream(prompt, system_prompt, Model.GEMINI, temperature)
        for chunk in response:
            yield chunk


import os
import json
import requests
from pathlib import Path
from rich.console import Console
from rich.table import Table
from service_forge.sft.util.logger import log_error, log_info, log_success, log_warning
from service_forge.sft.config.sft_config import sft_config

def remote_list_tars(service_center_url: str = None) -> None:
    """
    Get remote tar package list and status from service-center
    """
    # If URL is not provided, try to get it from configuration
    if not service_center_url:
        service_center_url = getattr(sft_config, 'service_center_address', 'http://localhost:5000')
    
    # Ensure URL ends with /
    if not service_center_url.endswith('/'):
        service_center_url += '/'
    
    api_url = f"{service_center_url}api/v1/services/tar-list"
    
    log_info(f"Getting tar package list from {api_url}...")
    
    try:
        # 发送GET请求
        response = requests.get(api_url, timeout=30)
        
        if response.status_code != 200:
            log_error(f"Failed to get tar package list, status code: {response.status_code}")
            try:
                error_data = response.json()
                log_error(f"Error message: {error_data.get('message', 'Unknown error')}")
            except:
                log_error(f"Response content: {response.text}")
            return
        
        # Parse response data
        result = response.json()
        
        if result.get('code') != 200:
            log_error(f"Failed to get tar package list: {result.get('message', 'Unknown error')}")
            return
        
        tar_files = result.get('data', [])
        
        if not tar_files:
            log_info("No tar packages found")
            return
        
        # Use rich table to display results
        console = Console()
        table = Table(title="Remote Server Tar Package List", show_header=True, header_style="bold magenta")
        table.add_column("Filename", style="cyan", no_wrap=True)
        table.add_column("Service Name", style="green", no_wrap=True)
        table.add_column("Version", style="blue", no_wrap=True)
        table.add_column("Size", justify="right", style="yellow")
        table.add_column("Modified Time", style="dim")
        table.add_column("Deploy Status", justify="center", style="bold")
        
        for tar_file in tar_files:
            # Format file size
            size = _format_size(tar_file.get('file_size', 0))
            
            # Format modification time
            modified_time = _format_time(tar_file.get('modified_time', 0))
            
            # Deployment status
            deployed_status = "✅ Deployed" if tar_file.get('deployed_status', False) else "❌ Not Deployed"
            status_style = "green" if tar_file.get('deployed_status', False) else "red"
            
            table.add_row(
                tar_file.get('filename', '-'),
                tar_file.get('service_name', '-'),
                tar_file.get('version', '-'),
                size,
                modified_time,
                f"[{status_style}]{deployed_status}[/{status_style}]"
            )
        
        console.print(table)
        log_success(f"Found {len(tar_files)} tar packages in total")
        
    except requests.exceptions.RequestException as e:
        log_error(f"Request failed: {str(e)}")
        log_info(f"Please check if service-center service is running normally and if the URL is correct: {service_center_url}")
    except Exception as e:
        log_error(f"Exception occurred while getting tar package list: {str(e)}")

def _format_size(size_bytes: int) -> str:
    """Format file size"""
    if size_bytes == 0:
        return "0 B"
    
    for unit in ['B', 'KB', 'MB', 'GB']:
        if size_bytes < 1024.0:
            return f"{size_bytes:.2f} {unit}"
        size_bytes /= 1024.0
    return f"{size_bytes:.2f} TB"

def _format_time(timestamp: float) -> str:
    """Format timestamp"""
    if timestamp == 0:
        return "-"
    
    try:
        from datetime import datetime
        return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")
    except:
        return "-"


from __future__ import annotations

from typing import TYPE_CHECKING
from ..utils.workflow_clone import edge_clone

if TYPE_CHECKING:
    from .node import Node
    from .port import Port

class Edge:
    def __init__(
        self,
        start_node: Node,
        end_node: Node,
        start_port: Port,
        end_port: Port,
    ) -> None:
        self.start_node = start_node
        self.end_node = end_node
        self.start_port = start_port
        self.end_port = end_port

    def _clone(self, node_map: dict[Node, Node], port_map: dict[Port, Port]) -> Edge:
        return edge_clone(self, node_map, port_map)



"""add description and example

Revision ID: 5fbbb21acc92
Revises: 8ff25e2b3fd9
Create Date: 2025-08-26 09:47:13.181504

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '5fbbb21acc92'
down_revision: Union[str, Sequence[str], None] = '8ff25e2b3fd9'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    op.add_column('tag', sa.Column('description', sa.String(255), nullable=True))
    op.add_column('tag', sa.Column('example', sa.String(255), nullable=True))


def downgrade() -> None:
    """Downgrade schema."""
    op.drop_column('tag', 'description')
    op.drop_column('tag', 'example')



import asyncio
from service_forge.api.kafka_api import KafkaApp
from pydantic import BaseModel

class FooInput(BaseModel):
    user_id: int
    data: str

app = KafkaApp("localhost:9092")

@app.kafka_input("test_topic")
async def handle_message(input: FooInput, metadata: dict):
    print(f"Received message: {input}, metadata: {metadata}")

if __name__ == "__main__":
    asyncio.run(app.start())



import json
import asyncio
import argparse
from aiokafka import AIOKafkaProducer

async def send_message(bootstrap_servers: str, topic: str, message: dict):
    producer = AIOKafkaProducer(
        bootstrap_servers=bootstrap_servers,
        value_serializer=lambda v: json.dumps(v).encode("utf-8"),
    )
    await producer.start()
    try:
        await producer.send_and_wait(topic, message)
        print(f"✅ 已发送到 {topic}: {message}")
    finally:
        await producer.stop()

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--topic", type=str, default="tag.request")
    parser.add_argument("--message", type=str, default="kafka message!")
    args = parser.parse_args()

    asyncio.run(send_message("localhost:9092", args.topic, {
        "proto_id": "123",
        "id": 34,
        "name": "example name",
        "text": "我明天早上八点有个会议",
        "type": "example type",
        "user_id": 58,
        "group_id": 10,
        "file_id": "example file id",
        "is_recognized": True,
        "is_deleted": False,
        "timestamp": "2021-01-01 00:00:00",
        "role": "user",
        "is_stream": True,
        "is_stream_done": False,
        "force_reply": False,
        "tags": [],
        "status": "",
    }))
